@@ -22,11 +22,9 @@
  * SOFTWARE.
  */
 use anyhow::{anyhow, bail, Result};
-use reqwest::blocking::Client;
-use reqwest::StatusCode;
 use serde_json::Value;
+use ureq::{Agent, AgentBuilder};
 // use std::io::Write;
-use custom_error::custom_error;
 use rand::seq::SliceRandom;
 use std::time::Duration;
 
@@ -42,7 +40,8 @@ const INVIDIOUS_INSTANCE_LIST: [&str; 7] = [
 
 pub struct InvidiousInstance {
     pub domain: Option<String>,
-    client: Client,
+    client: Agent,
+    query: Option<String>,
 }
 
 pub struct YoutubeVideo {
@@ -53,20 +52,22 @@ pub struct YoutubeVideo {
 
 impl Default for InvidiousInstance {
     fn default() -> Self {
-        let client = Client::default();
+        let client = Agent::new();
         let domain = Some(String::new());
+        let query = Some(String::new());
 
-        Self { domain, client }
+        Self {
+            domain,
+            client,
+            query,
+        }
     }
 }
 
-#[allow(unused_assignments)]
+#[allow(unused)]
 impl InvidiousInstance {
     pub fn new(query: &str) -> Result<(Self, Vec<YoutubeVideo>)> {
-        let mut client = Client::default();
-        if let Ok(c) = Client::builder().timeout(Duration::from_secs(10)).build() {
-            client = c;
-        }
+        let client = AgentBuilder::new().timeout(Duration::from_secs(10)).build();
 
         let mut domain = String::new();
         let mut domains = INVIDIOUS_INSTANCE_LIST;
@@ -74,51 +75,62 @@ impl InvidiousInstance {
         domains.shuffle(&mut rand::thread_rng());
         for v in domains.iter() {
             let mut url: String = v.to_string();
-            // url.push_str("/api/v1/stats");
-            url.push_str("/api/v1/search?q=");
-            url.push_str(query);
-            url.push_str("&page=");
-            url.push_str(&1.to_string());
+            url.push_str("/api/v1/search");
 
-            if let Ok(result) = client.get(&url).send() {
-                if result.status() == StatusCode::OK {
-                    if let Ok(text) = result.text() {
-                        if let Ok(vr) = InvidiousInstance::parse_youtube_options(text) {
+            if let Ok(result) = client.get(&url).query("q", query).query("page", "1").call() {
+                if result.status() == 200 {
+                    if let Ok(text) = result.into_string() {
+                        if let Some(vr) = InvidiousInstance::parse_youtube_options(&text) {
                             video_result = vr;
                             domain = v.to_string();
                             break;
-                        };
+                        }
                     }
                 }
             }
         }
         if domain.len() < 2 {
-            bail!("All invidious servers are down? Please check your network connection first.");
+            bail!("All 7 invidious servers are down? Please check your network connection first.");
         }
 
         let domain = Some(domain);
-        Ok((Self { domain, client }, video_result))
+        Ok((
+            Self {
+                domain,
+                client,
+                query: Some(query.to_string()),
+            },
+            video_result,
+        ))
     }
 
     // GetSearchQuery fetches query result from an Invidious instance.
-    pub fn get_search_query(&self, query: &str, page: u32) -> Result<Vec<YoutubeVideo>> {
+    pub fn get_search_query(&self, page: u32) -> Result<Vec<YoutubeVideo>> {
         let mut url = String::new();
         if let Some(u) = &self.domain {
             url.push_str(u);
         }
-        url.push_str("/api/v1/search?q=");
-        url.push_str(query);
-        url.push_str("&page=");
-        url.push_str(&page.to_string());
+        url.push_str("/api/v1/search");
 
-        let result = self.client.get(&url).send()?;
+        let query = match &self.query {
+            Some(q) => q,
+            None => bail!("No query string found"),
+        };
+
+        let result = self
+            .client
+            .get(&url)
+            .query("q", query)
+            .query("page", &page.to_string())
+            .call()?;
 
         match result.status() {
-            StatusCode::OK => match result.text() {
-                Ok(text) => InvidiousInstance::parse_youtube_options(text),
-                _ => Err(anyhow!("Error during search")),
+            200 => match result.into_string() {
+                Ok(text) => InvidiousInstance::parse_youtube_options(&text)
+                    .ok_or_else(|| anyhow!("None Error")),
+                Err(e) => bail!("Error during search: {}", e),
             },
-            _ => Err(anyhow!("Error during search")),
+            _ => bail!("Error during search"),
         }
     }
 
@@ -131,13 +143,14 @@ impl InvidiousInstance {
             "http://suggestqueries.google.com/complete/search?client=firefox&ds=yt&q=".to_string();
         url.push_str(prefix);
 
-        let result = self.client.get(&url).send()?;
+        let result = self.client.get(&url).call()?;
         match result.status() {
-            StatusCode::OK => match result.text() {
-                Ok(text) => InvidiousInstance::parse_youtube_options(text),
-                _ => Err(anyhow!("Error during search")),
+            200 => match result.into_string() {
+                Ok(text) => InvidiousInstance::parse_youtube_options(&text)
+                    .ok_or_else(|| anyhow!("None Error")),
+                Err(e) => bail!("Error during search: {}", e),
             },
-            _ => Err(anyhow!("Error during search")),
+            _ => bail!("Error during search"),
         }
 
         // res := []json.RawMessage{}
@@ -166,56 +179,35 @@ impl InvidiousInstance {
         url.push_str("type=music&region=");
         url.push_str(region);
 
-        let result = self.client.get(&url).send()?;
+        let result = self.client.get(&url).call()?;
 
         match result.status() {
-            StatusCode::OK => match result.text() {
-                Ok(text) => InvidiousInstance::parse_youtube_options(text),
-                _ => Err(anyhow!("Error during search")),
+            200 => match result.into_string() {
+                Ok(text) => InvidiousInstance::parse_youtube_options(&text)
+                    .ok_or_else(|| anyhow!("None Error")),
+                _ => bail!("Error during search"),
             },
-            _ => Err(anyhow!("Error during search")),
+            _ => bail!("Error during search"),
         }
     }
 
-    pub fn parse_youtube_options(data: String) -> Result<Vec<YoutubeVideo>> {
-        let value = serde_json::from_str::<Value>(&data)?;
-        let mut vec: Vec<YoutubeVideo> = Vec::new();
-        // let mut file = std::fs::File::create("data.txt").expect("create failed");
-        // file.write_all(data.as_bytes()).expect("write failed");
-        if let Some(array) = value.as_array() {
-            for v in array.iter() {
-                vec.push(YoutubeVideo {
-                    title: v
-                        .get("title")
-                        .ok_or(Errors::NoneError)?
-                        .as_str()
-                        .ok_or(Errors::NoneError)?
-                        .to_owned(),
-                    video_id: v
-                        .get("videoId")
-                        .ok_or(Errors::NoneError)?
-                        .as_str()
-                        .ok_or(Errors::NoneError)?
-                        .to_owned(),
-                    length_seconds: v
-                        .get("lengthSeconds")
-                        .ok_or(Errors::NoneError)?
-                        .as_u64()
-                        .ok_or(Errors::NoneError)? as u64,
-                })
+    pub fn parse_youtube_options(data: &str) -> Option<Vec<YoutubeVideo>> {
+        if let Ok(value) = serde_json::from_str::<Value>(data) {
+            let mut vec: Vec<YoutubeVideo> = Vec::new();
+            // below two lines are left for debug purpose
+            // let mut file = std::fs::File::create("data.txt").expect("create failed");
+            // file.write_all(data.as_bytes()).expect("write failed");
+            if let Some(array) = value.as_array() {
+                for v in array.iter() {
+                    vec.push(YoutubeVideo {
+                        title: v.get("title")?.as_str()?.to_owned(),
+                        video_id: v.get("videoId")?.as_str()?.to_owned(),
+                        length_seconds: v.get("lengthSeconds")?.as_u64()?,
+                    })
+                }
+                return Some(vec);
             }
         }
-        Ok(vec)
+        None
     }
 }
-
-custom_error! { pub Errors
-    OpenSSLError{ source: openssl::error::ErrorStack } = "openSSL Error",
-    RegexError{ source: regex::Error } = "regex Error",
-    SerdeJsonError{ source: serde_json::error::Error } = "serde json Error",
-    ParseError{ source: std::num::ParseIntError } = "parse Error",
-    // AsyncIoError{ source: io::Error } = "async io Error",
-    // IsahcError{ source: isahc::Error } = "isahc Error",
-    NoneError = "None Error",
-    // FromUtf8Error{source: std::string::FromUtf8Error} = "UTF8 Error",
-}
