@@ -23,31 +23,18 @@
  */
 mod app;
 mod config;
-pub mod invidious;
+mod invidious;
 mod player;
 mod song;
 mod songtag;
 mod ui;
 
-use anyhow::{anyhow, Result};
 use app::App;
 use config::TermusicConfig;
-use configr::Config;
 
-fn main() -> Result<()> {
-    let mut path = dirs_next::home_dir()
-        .map(|h| h.join(".config"))
-        .ok_or_else(|| anyhow!("failed to find os config dir."))?;
-
-    let config = match TermusicConfig::load("termusic", true) {
-        Ok(c) => c,
-        Err(_) => match TermusicConfig::load_custom("termusic", &mut path) {
-            Ok(c) => c,
-            Err(_) => TermusicConfig::default(),
-        },
-    };
+fn main() {
+    let config = TermusicConfig::load().unwrap_or_default();
 
     let mut app: App = App::new(config);
     app.run();
-    Ok(())
 }
