@@ -25,12 +25,14 @@ use crate::player::gst::GSTPlayer;
 use crate::songtag::lrc::Lyric;
 use anyhow::{anyhow, Result};
 use humantime::{format_duration, FormattedDuration};
-use id3::frame::Lyrics;
-use id3::frame::{Picture, PictureType};
+use id3::frame::{Lyrics, Picture, PictureType};
+use metaflac::Tag as FlacTag;
+// use lofty::{self, AudioTag, MimeType, Picture, Tag as LoftyTag};
 use mp4ameta::{Img, ImgFmt};
+// use std::cmp::Ordering;
 use std::ffi::OsStr;
 use std::fs::rename;
-use std::path::{Path, PathBuf};
+use std::path::Path;
 use std::str::FromStr;
 use std::time::Duration;
 
@@ -45,7 +47,7 @@ pub struct Song {
     /// File path to the song
     pub file: Option<String>,
     /// Duration of the song
-    pub duration: Option<Duration>,
+    pub duration: Duration,
     /// Name of the song
     pub name: Option<String>,
     /// Extension of the song
@@ -92,14 +94,31 @@ impl Song {
     }
 
     pub fn duration(&self) -> FormattedDuration {
-        match self.duration.as_ref() {
-            Some(d) => format_duration(Duration::from_secs(d.as_secs())),
-            None => format_duration(Duration::from_secs(0)),
+        format_duration(self.duration)
+    }
+    pub fn update_duration(&self) {
+        if let Some(s) = &self.file() {
+            if let Some(ext) = &self.ext {
+                match ext.as_str() {
+                    "mp3" => {
+                        let mut id3_tag = id3::Tag::new();
+                        if let Ok(t) = id3::Tag::read_from_path(s) {
+                            id3_tag = t;
+                        }
+
+                        let duration_player = GSTPlayer::duration(s);
+                        id3_tag.remove_duration();
+                        id3_tag.set_duration(duration_player.mseconds() as u32);
+                        let _ = id3_tag.write_to_path(s, id3::Version::Id3v24);
+                    }
+                    &_ => {}
+                }
+            }
         }
     }
 
-    pub fn save(&mut self) -> Result<()> {
-        if let Some(ext) = self.ext.as_ref() {
+    pub fn save_tag(&mut self) -> Result<()> {
+        if let Some(ext) = &self.ext {
             match ext.as_str() {
                 "mp3" => {
                     let mut id3_tag = id3::Tag::default();
@@ -115,7 +134,7 @@ impl Song {
                     id3_tag.remove_all_lyrics();
 
                     if !self.lyric_frames.is_empty() {
-                        let lyric_frames = self.lyric_frames.clone();
+                        let lyric_frames = self.lyric_frames.to_owned();
                         for l in lyric_frames {
                             id3_tag.add_lyrics(l);
                         }
@@ -148,17 +167,25 @@ impl Song {
                     m4a_tag.remove_lyrics();
 
                     if !self.lyric_frames.is_empty() {
-                        let lyric_frames = self.lyric_frames.clone();
+                        let lyric_frames = self.lyric_frames.to_owned();
                         for l in lyric_frames {
                             m4a_tag.set_lyrics(l.text);
                         }
                     }
 
                     for p in self.picture.iter() {
+                        let fmt = match p.mime_type.as_str() {
+                            "image/jpeg" => ImgFmt::Jpeg,
+                            "image/bmp" => ImgFmt::Bmp,
+                            "image/Png" => ImgFmt::Png,
+                            &_ => ImgFmt::Jpeg,
+                        };
+
                         let img = Img {
                             data: p.data.to_owned(),
-                            fmt: ImgFmt::Jpeg,
+                            fmt,
                         };
+
                         m4a_tag.set_artwork(img);
                     }
 
@@ -168,11 +195,60 @@ impl Song {
                             .map_err(|e| anyhow!("write m4a tag error {:?}", e))?;
                     }
                 }
+                "flac" => {
+                    let mut flac_tag = FlacTag::default();
+                    if let Some(file) = self.file() {
+                        if let Ok(t) = FlacTag::read_from_path(file) {
+                            flac_tag = t;
+                        }
+                    }
+
+                    flac_tag.set_vorbis(
+                        "Artist",
+                        vec![self.artist().unwrap_or(&String::from("Unknown Artist"))],
+                    );
+                    flac_tag.set_vorbis(
+                        "Title",
+                        vec![self.title().unwrap_or(&String::from("Unknown Title"))],
+                    );
+                    flac_tag.set_vorbis(
+                        "Album",
+                        vec![self
+                            .album
+                            .as_ref()
+                            .unwrap_or(&String::from("Unknown Album"))],
+                    );
+                    flac_tag.remove_vorbis("Lyrics");
+
+                    if !self.lyric_frames.is_empty() {
+                        let lyric_frames = self.lyric_frames.to_owned();
+                        for l in lyric_frames {
+                            flac_tag.set_vorbis("Lyrics", vec![l.text]);
+                        }
+                    }
+
+                    let pictures = self.picture.clone();
+                    for p in pictures.into_iter() {
+                        flac_tag.add_picture(
+                            p.mime_type,
+                            metaflac::block::PictureType::Other,
+                            p.data,
+                        );
+                    }
+
+                    if let Some(file) = self.file() {
+                        let _ = flac_tag
+                            .write_to_path(file)
+                            .map_err(|e| anyhow!("write m4a tag error {:?}", e))?;
+                    }
+                }
 
                 &_ => {}
             }
         }
 
+        self.rename_by_tag()?;
+
         Ok(())
     }
 
@@ -188,30 +264,35 @@ impl Song {
             let p_old: &Path = Path::new(file);
             if let Some(p_prefix) = p_old.parent() {
                 let p_new = p_prefix.join(new_name_path);
-                rename(p_old, <PathBuf as AsRef<Path>>::as_ref(&p_new))
-                    .map_err(|e| anyhow!("rename m4a file error {:?}", e))?;
+                rename(p_old, &p_new)?;
                 self.file = Some(String::from(p_new.to_string_lossy()));
-                if let Some(name) = p_new.file_name() {
-                    self.name = Some(String::from(name.to_string_lossy()));
-                }
+                self.name = p_new
+                    .file_name()
+                    .and_then(OsStr::to_str)
+                    .map(|x| x.to_string());
             }
         }
         Ok(())
     }
 
-    pub fn set_lyric(&mut self, lyric_string: &str, lang_ext: &str) {
-        let mut lyric_frames = self.lyric_frames.clone();
-        match self.lyric_frames.get_mut(self.lyric_selected) {
+    pub fn set_lyric(&mut self, lyric_str: &str, lang_ext: &str) {
+        let mut lyric_frames = self.lyric_frames.to_owned();
+        match self.lyric_frames.get(self.lyric_selected) {
             Some(lyric_frame) => {
-                lyric_frame.text = lyric_string.to_string();
                 lyric_frames.remove(self.lyric_selected);
-                lyric_frames.insert(self.lyric_selected, lyric_frame.clone());
+                lyric_frames.insert(
+                    self.lyric_selected,
+                    Lyrics {
+                        text: lyric_str.to_string(),
+                        ..lyric_frame.to_owned()
+                    },
+                );
             }
             None => {
                 lyric_frames.push(Lyrics {
-                    lang: lang_ext.to_string(),
-                    description: String::from("Termusic"),
-                    text: lyric_string.to_string(),
+                    lang: "eng".to_string(),
+                    description: lang_ext.to_string(),
+                    text: lyric_str.to_string(),
                 });
             }
         }
@@ -225,7 +306,8 @@ impl Song {
 }
 
 impl FromStr for Song {
-    type Err = std::string::ParseError;
+    type Err = anyhow::Error;
+    // type Err = std::string::ParseError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         let p: &Path = Path::new(s);
@@ -234,10 +316,6 @@ impl FromStr for Song {
         match ext {
             Some("mp3") => {
                 let name = p.file_name().and_then(OsStr::to_str).map(|x| x.to_string());
-                let duration: Option<Duration> = match mp3_duration::from_path(s) {
-                    Ok(d) => Some(d),
-                    Err(_) => Some(Duration::from_secs(0)),
-                };
 
                 let id3_tag = match id3::Tag::read_from_path(s) {
                     Ok(tag) => tag,
@@ -259,7 +337,7 @@ impl FromStr for Song {
                 for l in id3_tag.lyrics().cloned() {
                     lyrics.push(l);
                 }
-                lyrics.sort_by(|b, a| b.description.cmp(&a.description));
+                lyrics.sort_by_cached_key(|a| a.description.to_owned());
 
                 let mut parsed_lyric: Option<Lyric> = None;
                 if !lyrics.is_empty() {
@@ -274,7 +352,13 @@ impl FromStr for Song {
                     picture.push(p);
                 }
 
+                let duration = id3_tag
+                    .duration()
+                    .map(|d| Duration::from_millis(d.into()))
+                    .unwrap_or_else(|| Duration::from_secs(0));
+
                 let file = Some(String::from(s));
+
                 Ok(Self {
                     artist,
                     album,
@@ -282,7 +366,7 @@ impl FromStr for Song {
                     file,
                     duration,
                     name,
-                    ext: Some("mp3".to_string()),
+                    ext: ext.map(String::from),
                     lyric_frames: lyrics,
                     lyric_selected: 0,
                     parsed_lyric,
@@ -292,8 +376,6 @@ impl FromStr for Song {
             Some("m4a") => {
                 let name = p.file_name().and_then(OsStr::to_str).map(|x| x.to_string());
 
-                let duration_u64 = GSTPlayer::duration_m4a(s);
-                let duration = Some(Duration::from_secs(duration_u64));
                 let m4a_tag = match mp4ameta::Tag::read_from_path(s) {
                     Ok(t) => t,
                     Err(_) => {
@@ -302,10 +384,8 @@ impl FromStr for Song {
                         if let Some(p_base) = p.file_stem() {
                             t.set_title(p_base.to_string_lossy());
                         }
-                        match t.write_to_path(p) {
-                            Ok(_) => t,
-                            Err(_) => t,
-                        }
+                        let _ = t.write_to_path(p);
+                        t
                     }
                 };
 
@@ -346,6 +426,8 @@ impl FromStr for Song {
                     });
                 }
 
+                let duration = m4a_tag.duration().unwrap_or_else(|| Duration::from_secs(0));
+
                 let file = Some(String::from(s));
                 Ok(Self {
                     artist,
@@ -354,7 +436,118 @@ impl FromStr for Song {
                     file,
                     duration,
                     name,
-                    ext: Some("m4a".to_string()),
+                    ext: ext.map(String::from),
+                    lyric_frames,
+                    lyric_selected: 0,
+                    parsed_lyric,
+                    picture,
+                })
+            }
+            Some("flac") => {
+                let name = p.file_name().and_then(OsStr::to_str).map(|x| x.to_string());
+
+                let flac_tag = match FlacTag::read_from_path(s) {
+                    Ok(t) => t,
+                    Err(_) => {
+                        let mut t = FlacTag::default();
+                        let p: &Path = Path::new(s);
+                        if let Some(p_base) = p.file_stem() {
+                            t.set_vorbis("Title", vec![p_base.to_string_lossy()]);
+                        }
+                        let _ = t.write_to_path(p);
+                        t
+                    }
+                };
+
+                let artist: Option<String>;
+                let a_vec = flac_tag.get_vorbis("Artist");
+                match a_vec {
+                    Some(a_vec) => {
+                        let mut a_string = String::new();
+                        for a in a_vec.into_iter() {
+                            a_string.push_str(a);
+                        }
+                        artist = Some(a_string);
+                    }
+                    None => artist = None,
+                }
+
+                let album: Option<String>;
+                let album_vec = flac_tag.get_vorbis("Album");
+                match album_vec {
+                    Some(album_vec) => {
+                        let mut album_string = String::new();
+                        for a in album_vec.into_iter() {
+                            album_string.push_str(a);
+                        }
+                        album = Some(album_string);
+                    }
+                    None => album = None,
+                }
+
+                let title: Option<String>;
+                let title_vec = flac_tag.get_vorbis("Title");
+                match title_vec {
+                    Some(title_vec) => {
+                        let mut title_string = String::new();
+                        for t in title_vec.into_iter() {
+                            title_string.push_str(t);
+                        }
+                        title = Some(title_string);
+                    }
+                    None => title = None,
+                }
+
+                let mut lyric_frames: Vec<Lyrics> = vec![];
+
+                let lyric_vec = flac_tag.get_vorbis("Lyrics");
+                if let Some(l_vec) = lyric_vec {
+                    for l in l_vec.into_iter() {
+                        lyric_frames.push(Lyrics {
+                            lang: "eng".to_string(),
+                            description: "termusic".to_string(),
+                            text: l.to_string(),
+                        })
+                    }
+                }
+
+                let mut parsed_lyric: Option<Lyric> = None;
+                if let Some(l) = lyric_frames.get(0) {
+                    parsed_lyric = match Lyric::from_str(&l.text) {
+                        Ok(l) => Some(l),
+                        Err(_) => None,
+                    }
+                }
+
+                let mut picture: Vec<Picture> = Vec::new();
+                let picture_vec = flac_tag.pictures();
+                for p in picture_vec.into_iter() {
+                    picture.push(Picture {
+                        mime_type: p.mime_type.clone(),
+                        picture_type: PictureType::Other,
+                        description: "some image".to_string(),
+                        data: p.data.to_vec(),
+                    })
+                }
+
+                let mut duration = Duration::from_secs(0);
+                let stream_info = flac_tag.get_streaminfo();
+                if let Some(s) = stream_info {
+                    let secs = s.total_samples.checked_div(s.sample_rate as u64);
+                    if let Some(s) = secs {
+                        duration = Duration::from_secs(s);
+                    }
+                }
+
+                let file = Some(String::from(s));
+                Ok(Self {
+                    artist,
+                    album,
+                    title,
+                    file,
+                    duration,
+                    name,
+                    ext: ext.map(String::from),
                     lyric_frames,
                     lyric_selected: 0,
                     parsed_lyric,
@@ -366,7 +559,7 @@ impl FromStr for Song {
                 let album = Some(String::from("Not Support?"));
                 let title = Some(String::from(s));
                 let file = Some(String::from(s));
-                let duration = Some(Duration::from_secs(0));
+                let duration = Duration::from_secs(0);
                 let name = Some(String::from(""));
                 let parsed_lyric: Option<Lyric> = None;
                 let lyric_frames: Vec<Lyrics> = Vec::new();
@@ -378,7 +571,7 @@ impl FromStr for Song {
                     file,
                     duration,
                     name,
-                    ext: ext.map(|x| x.to_string()),
+                    ext: ext.map(String::from),
                     lyric_frames,
                     lyric_selected: 0,
                     parsed_lyric,
