@@ -25,18 +25,16 @@ mod kugou;
 pub mod lrc;
 mod migu;
 mod netease;
-use crate::ui::activity::main::TransferState;
-use crate::ui::activity::tageditor::SearchLyricState;
+use crate::ui::activity::{main::TransferState, tageditor::SearchLyricState};
 use anyhow::{anyhow, bail, Result};
-use id3::frame::Lyrics;
-use id3::frame::{Picture, PictureType};
-use id3::{Tag, Version};
+use id3::{
+    frame::{Lyrics, Picture, PictureType},
+    Tag, Version,
+};
 use serde::{Deserialize, Serialize};
 use std::path::{Path, PathBuf};
-use std::sync::mpsc;
-use std::sync::mpsc::{Receiver, Sender};
-use std::thread;
-use std::thread::sleep;
+use std::sync::mpsc::{self, Receiver, Sender};
+use std::thread::{self, sleep};
 use std::time::Duration;
 use ytd_rs::{Arg, ResultType, YoutubeDL};
 
@@ -61,8 +59,8 @@ pub enum SongtagProvider {
     Migu,
 }
 
-// Search function of 3 servers. Run parallel to get results faster.
-pub fn lyric_options(search_str: &str, tx_tageditor: Sender<SearchLyricState>) {
+// Search function of 3 servers. Run in parallel to get results faster.
+pub fn songtag_search(search_str: &str, tx_tageditor: Sender<SearchLyricState>) {
     let mut results: Vec<SongTag> = Vec::new();
     let (tx, rx): (Sender<Vec<SongTag>>, Receiver<Vec<SongTag>>) = mpsc::channel();
 
@@ -119,11 +117,31 @@ pub fn lyric_options(search_str: &str, tx_tageditor: Sender<SearchLyricState>) {
 
         let _ = tx_tageditor.send(SearchLyricState::Finish(results));
     });
-
-    // Ok(results)
 }
 
 impl SongTag {
+    pub fn artist(&self) -> Option<&str> {
+        match self.artist.as_ref() {
+            Some(artist) => Some(artist),
+            None => None,
+        }
+    }
+
+    pub fn album(&self) -> Option<&str> {
+        match self.album.as_ref() {
+            Some(album) => Some(album),
+            None => None,
+        }
+    }
+    /// Optionally return the title of the song
+    /// If `None` it wasn't able to read the tags
+    pub fn title(&self) -> Option<&str> {
+        match self.title.as_ref() {
+            Some(title) => Some(title),
+            None => None,
+        }
+    }
+
     // get lyric by lyric_id
     pub fn fetch_lyric(&self) -> Result<String> {
         let mut lyric_string = String::new();
@@ -243,12 +261,12 @@ impl SongTag {
         match self.service_provider {
             Some(SongtagProvider::Netease) => {
                 let mut netease_api = netease::NeteaseApi::new();
-                url = netease_api.song_url(song_id.to_string())?;
+                url = netease_api.song_url(song_id)?;
             }
             Some(SongtagProvider::Migu) => {}
             Some(SongtagProvider::Kugou) => {
                 let mut kugou_api = kugou::KugouApi::new();
-                url = kugou_api.song_url(song_id.to_string(), album_id)?;
+                url = kugou_api.song_url(song_id, &album_id)?;
             }
             None => url = String::new(),
         }
@@ -268,12 +286,12 @@ impl SongTag {
             // check what the result is and print out the path to the download or the error
             match download.result_type() {
                 ResultType::SUCCESS => {
-                    let mut tag_song = Tag::new();
-                    tag_song.set_album(album);
-                    tag_song.set_title(title);
-                    tag_song.set_artist(artist);
+                    let mut song_id3tag = Tag::new();
+                    song_id3tag.set_album(album);
+                    song_id3tag.set_title(title);
+                    song_id3tag.set_artist(artist);
                     if let Ok(l) = lyric {
-                        tag_song.add_lyrics(Lyrics {
+                        song_id3tag.add_lyrics(Lyrics {
                             lang: String::from("chi"),
                             description: String::from("saved by termusic."),
                             text: l,
@@ -281,11 +299,11 @@ impl SongTag {
                     }
 
                     if let Ok(p) = photo {
-                        tag_song.add_picture(p);
+                        song_id3tag.add_picture(p);
                     }
 
                     let file = p_full.as_str();
-                    if tag_song.write_to_path(file, Version::Id3v24).is_ok() {
+                    if song_id3tag.write_to_path(file, Version::Id3v24).is_ok() {
                         let _ = tx.send(TransferState::Success);
                         sleep(Duration::from_secs(5));
                         let _ = tx.send(TransferState::Completed(Some(p_full)));
