@@ -36,33 +36,30 @@ mod view;
 mod youtube_options;
 
 // Locals
-// use super::super::super::player::Player;
 use super::{Activity, Context, ExitReason, Status};
-use crate::config::MUSIC_DIR;
-use crate::song::Song;
-use crate::ui::activity::tageditor::TagEditorActivity;
+use crate::{
+    config::{TermusicConfig, MUSIC_DIR},
+    player::{AudioPlayer, Player},
+    song::Song,
+    ui::activity::tageditor::TagEditorActivity,
+};
 use std::str::FromStr;
 // Ext
-use crate::config::TermusicConfig;
-use crate::player::AudioPlayer;
-use crate::player::Player;
 use crossterm::terminal::{disable_raw_mode, enable_raw_mode};
 use log::error;
 use std::path::{Path, PathBuf};
-use std::sync::mpsc;
-use std::sync::mpsc::{Receiver, Sender};
+use std::sync::mpsc::{self, Receiver, Sender};
 use std::thread::sleep;
 use std::time::Duration;
 use tui_realm_treeview::Tree;
-// use tuirealm::event::{Event, KeyCode, KeyEvent, KeyModifiers};
 use tuirealm::{Payload, Value, View};
-use youtube_options::YoutubeOptions;
+use youtube_options::{YoutubeOptions, YoutubeSearchState};
 
 // -- components
 const COMPONENT_LABEL_HELP: &str = "LABEL_HELP";
 const COMPONENT_PARAGRAPH_LYRIC: &str = "PARAGRAPH_LYRIC";
-const COMPONENT_TABLE: &str = "SCROLLTABLE";
-const COMPONENT_SCROLLTABLE_YOUTUBE: &str = "SCROLLTABLE_YOUTUBE";
+const COMPONENT_TABLE_QUEUE: &str = "SCROLLTABLE_QUEUE";
+const COMPONENT_TABLE_YOUTUBE: &str = "SCROLLTABLE_YOUTUBE";
 const COMPONENT_TREEVIEW: &str = "TREEVIEW";
 const COMPONENT_PROGRESS: &str = "PROGRESS";
 const COMPONENT_TEXT_HELP: &str = "TEXT_HELP";
@@ -97,12 +94,15 @@ pub struct MainActivity {
     youtube_options: YoutubeOptions,
     sender_message: Sender<MessageState>,
     receiver_message: Receiver<MessageState>,
+    sender_youtubesearch: Sender<YoutubeSearchState>,
+    receiver_youtubesearch: Receiver<YoutubeSearchState>,
 }
 
 pub enum MessageState {
     Show((String, String)),
     Hide,
 }
+
 // TransferState is used to describe the status of download
 pub enum TransferState {
     Running, // indicates progress
@@ -112,6 +112,7 @@ pub enum TransferState {
     ErrEmbedData,
 }
 
+// StatusLine shows the status of download
 pub enum StatusLine {
     Default,
     Success,
@@ -131,6 +132,8 @@ impl Default for MainActivity {
         let p: &Path = Path::new(full_path.as_ref());
         let (tx, rx): (Sender<TransferState>, Receiver<TransferState>) = mpsc::channel();
         let (tx2, rx2): (Sender<MessageState>, Receiver<MessageState>) = mpsc::channel();
+        let (tx3, rx3): (Sender<YoutubeSearchState>, Receiver<YoutubeSearchState>) =
+            mpsc::channel();
         MainActivity {
             exit_reason: None,
             context: None,
@@ -150,14 +153,16 @@ impl Default for MainActivity {
             youtube_options: YoutubeOptions::new(),
             sender_message: tx2,
             receiver_message: rx2,
+            sender_youtubesearch: tx3,
+            receiver_youtubesearch: rx3,
         }
     }
 }
 
 impl MainActivity {
-    pub fn init_config(&mut self, config: TermusicConfig) {
-        self.config = config;
-        let music_dir = self.config.music_dir.clone();
+    pub fn init_config(&mut self, config: &TermusicConfig) {
+        self.config = config.to_owned();
+        let music_dir = self.config.music_dir.to_owned();
         let full_path = shellexpand::tilde(&music_dir);
         let p: &Path = Path::new(full_path.as_ref());
         self.scan_dir(p);
@@ -173,10 +178,12 @@ impl MainActivity {
                 if let Some(file) = &song.file {
                     self.player.queue_and_play(file);
                 }
-                self.queue_items.push(song.clone());
+                self.queue_items.push(song.to_owned());
                 self.current_song = Some(song);
-                self.sync_items();
+                self.sync_queue();
                 self.update_photo();
+                self.update_progress_title();
+                self.update_duration();
             }
             Some(Status::Running) => {}
             Some(Status::Paused) => {}
@@ -226,7 +233,8 @@ impl MainActivity {
                 break;
             }
             if let Some(ExitReason::NeedRefreshPlaylist(file)) = tageditor.will_umount() {
-                self.refresh_playlist(Some(file));
+                // print!("{}", file);
+                self.sync_playlist(Some(file));
                 self.update_item_delete();
             }
 
