@@ -23,7 +23,7 @@
  */
 use super::{MainActivity, COMPONENT_TREEVIEW};
 use anyhow::{bail, Result};
-use pinyin::{to_pinyin_vec, Pinyin};
+use pinyin::ToPinyin;
 use std::fs::{remove_dir_all, remove_file, rename};
 use std::path::Path;
 use tui_realm_treeview::{Node, Tree, TreeViewPropsBuilder};
@@ -47,18 +47,15 @@ impl MainActivity {
         };
         let mut node: Node = Node::new(p.to_string_lossy().into_owned(), name);
         if depth > 0 && p.is_dir() {
-            let mut paths: Vec<_> = std::fs::read_dir(p)
-                .unwrap()
-                .filter_map(|r| r.ok())
-                .collect();
-            // paths.sort_by_key(|dir| dir.path());
-            paths.sort_by(|a, b| {
-                get_pin_yin(&a.file_name().to_string_lossy().to_string())
-                    .cmp(&get_pin_yin(&b.file_name().to_string_lossy().to_string()))
-            });
+            if let Ok(paths) = std::fs::read_dir(p) {
+                let mut paths: Vec<_> = paths.filter_map(|r| r.ok()).collect();
 
-            for p in paths {
-                node.add_child(Self::dir_tree(p.path().as_path(), depth - 1));
+                paths.sort_by_cached_key(|k| {
+                    get_pin_yin(&k.file_name().to_string_lossy().to_string())
+                });
+                for p in paths {
+                    node.add_child(Self::dir_tree(p.path().as_path(), depth - 1));
+                }
             }
         }
         node
@@ -67,26 +64,23 @@ impl MainActivity {
     pub fn dir_children(p: &Path) -> Vec<String> {
         let mut children: Vec<String> = vec![];
         if p.is_dir() {
-            let mut paths: Vec<_> = std::fs::read_dir(p)
-                .unwrap()
-                .filter_map(|r| r.ok())
-                // .filter_map(|r| r.path().is_dir())
-                .collect();
-            paths.sort_by(|a, b| {
-                get_pin_yin(&a.file_name().to_string_lossy().to_string())
-                    .cmp(&get_pin_yin(&b.file_name().to_string_lossy().to_string()))
-            });
+            if let Ok(paths) = std::fs::read_dir(p) {
+                let mut paths: Vec<_> = paths.filter_map(|r| r.ok()).collect();
 
-            for p in paths {
-                if !p.path().is_dir() {
-                    children.push(String::from(p.path().to_string_lossy()));
+                paths.sort_by_cached_key(|k| {
+                    get_pin_yin(&k.file_name().to_string_lossy().to_string())
+                });
+                for p in paths {
+                    if !p.path().is_dir() {
+                        children.push(String::from(p.path().to_string_lossy()));
+                    }
                 }
             }
         }
         children
     }
 
-    pub fn refresh_playlist(&mut self, node: Option<&str>) {
+    pub fn sync_playlist(&mut self, node: Option<&str>) {
         self.tree = Tree::new(Self::dir_tree(self.path.as_ref(), 3));
 
         if let Some(props) = self.view.get_props(COMPONENT_TREEVIEW) {
@@ -117,7 +111,7 @@ impl MainActivity {
 
             self.view.on(event);
 
-            self.refresh_playlist(None);
+            self.sync_playlist(None);
         }
 
         // this line remove the deleted songs from queue
@@ -138,7 +132,7 @@ impl MainActivity {
             });
             self.view.on(event);
 
-            self.refresh_playlist(None);
+            self.sync_playlist(None);
         }
 
         // this line remove the deleted songs from queue
@@ -153,18 +147,19 @@ impl MainActivity {
     }
 
     pub fn paste(&mut self) -> Result<()> {
-        if let Some(Payload::One(Value::Str(node_id))) = self.view.get_state(COMPONENT_TREEVIEW) {
+        if let Some(Payload::One(Value::Str(new_id))) = self.view.get_state(COMPONENT_TREEVIEW) {
             match self.yanked_node_id.as_ref() {
-                Some(id) => {
-                    let p: &Path = Path::new(node_id.as_str());
-                    let pold: &Path = Path::new(id.as_str());
+                Some(old_id) => {
+                    let p: &Path = Path::new(new_id.as_str());
+                    let pold: &Path = Path::new(old_id.as_str());
                     if let Some(p_parent) = p.parent() {
                         if let Some(pold_filename) = pold.file_name() {
                             let mut new_node_id = p_parent.join(pold_filename);
                             if p.is_dir() {
                                 new_node_id = p.join(pold_filename);
                             }
-                            rename(pold, new_node_id)?;
+                            rename(pold, new_node_id.as_path())?;
+                            self.sync_playlist(new_node_id.to_str());
                         }
                     }
                 }
@@ -172,19 +167,39 @@ impl MainActivity {
             }
         }
         self.yanked_node_id = None;
-        self.refresh_playlist(None);
         self.update_item_delete();
         Ok(())
     }
 }
 
-fn get_pin_yin(parma: &str) -> String {
-    let a = to_pinyin_vec(parma, Pinyin::plain).join("");
-    let mut b = a;
-    // let mut temp: String = String::new();
-    if b.is_empty() {
-        let temp = parma.to_lowercase();
-        b = temp;
+fn get_pin_yin(input: &str) -> String {
+    let mut b = String::new();
+    for (index, f) in input.to_pinyin().enumerate() {
+        match f {
+            Some(p) => {
+                b.push_str(p.plain());
+            }
+            None => {
+                if let Some(c) = input.to_uppercase().chars().nth(index) {
+                    b.push(c);
+                }
+            }
+        }
     }
     b
 }
+
+#[cfg(test)]
+mod tests {
+
+    use crate::ui::activity::main::playlist::get_pin_yin;
+    use pretty_assertions::assert_eq;
+
+    #[test]
+    fn test_pin_yin() {
+        assert_eq!(get_pin_yin("陈一发儿"), "chenyifaer".to_string());
+        assert_eq!(get_pin_yin("Gala乐队"), "GALAledui".to_string());
+        assert_eq!(get_pin_yin("乐队Gala乐队"), "leduiGALAledui".to_string());
+        assert_eq!(get_pin_yin("Annett Louisan"), "ANNETT LOUISAN".to_string());
+    }
+}
