@@ -22,21 +22,21 @@
  * SOFTWARE.
  */
 use super::MainActivity;
-use super::{COMPONENT_TABLE, COMPONENT_TREEVIEW};
+use super::{COMPONENT_TABLE_QUEUE, COMPONENT_TREEVIEW};
 
+use crate::config::get_app_config_path;
 use crate::song::Song;
-use crate::ui::components::table;
-use anyhow::{anyhow, bail, Result};
+use anyhow::{bail, Result};
 use humantime::format_duration;
 use rand::seq::SliceRandom;
 use rand::thread_rng;
-use std::fs::{self, File};
+use std::fs::File;
 use std::io::{BufRead, BufReader, Write};
-use std::path::{Path, PathBuf};
+use std::path::Path;
 use std::str::FromStr;
 use std::time::Duration;
+use tui_realm_stdlib::TablePropsBuilder;
 use tuirealm::PropsBuilder;
-use unicode_truncate::{Alignment, UnicodeTruncateStr};
 
 use tuirealm::props::{TableBuilder, TextSpan};
 
@@ -44,10 +44,10 @@ impl MainActivity {
     pub fn add_queue(&mut self, item: Song) {
         self.queue_items.insert(0, item);
 
-        self.sync_items();
+        self.sync_queue();
     }
 
-    pub fn sync_items(&mut self) {
+    pub fn sync_queue(&mut self) {
         let mut table: TableBuilder = TableBuilder::default();
 
         for (idx, record) in self.queue_items.iter().enumerate() {
@@ -55,27 +55,21 @@ impl MainActivity {
                 table.add_row();
             }
 
-            let duration = record.duration();
-            let duration_string = format!("{}", duration);
-            let duration_truncated = duration_string.unicode_pad(6, Alignment::Left, true);
+            let mut duration = record.duration().to_string();
+            duration.truncate(6);
+            let duration_string = format!("[{:^6}]", duration);
 
             let name = record
                 .name
                 .to_owned()
                 .unwrap_or_else(|| "No Name".to_string());
             let artist = record.artist().unwrap_or(&name);
-            let artist_truncated = artist.unicode_pad(14, Alignment::Left, true);
             let title = record.title().unwrap_or("Unknown Title");
-            let title_truncated = title.unicode_pad(20, Alignment::Left, true);
 
             table
-                .add_col(TextSpan::new(
-                    format!("[{}] ", duration_truncated,).as_str(),
-                ))
-                .add_col(
-                    TextSpan::new(&artist_truncated).fg(tuirealm::tui::style::Color::LightYellow),
-                )
-                .add_col(TextSpan::new(title_truncated.as_ref()).bold())
+                .add_col(TextSpan::new(duration_string.as_str()))
+                .add_col(TextSpan::new(artist).fg(tuirealm::tui::style::Color::LightYellow))
+                .add_col(TextSpan::new(title).bold())
                 .add_col(TextSpan::new(record.album().unwrap_or("Unknown Album")));
         }
         if self.queue_items.is_empty() {
@@ -88,12 +82,13 @@ impl MainActivity {
         let table = table.build();
         let title = self.update_title();
 
-        if let Some(props) = self.view.get_props(COMPONENT_TABLE) {
-            let props = table::TablePropsBuilder::from(props)
+        if let Some(props) = self.view.get_props(COMPONENT_TABLE_QUEUE) {
+            let props = TablePropsBuilder::from(props)
                 .with_title(title, tuirealm::tui::layout::Alignment::Left)
                 .with_table(table)
                 .build();
-            self.view.update(COMPONENT_TABLE, props);
+            let msg = self.view.update(COMPONENT_TABLE_QUEUE, props);
+            self.update(msg);
         }
     }
     pub fn delete_item(&mut self, index: usize) {
@@ -101,22 +96,21 @@ impl MainActivity {
             return;
         }
         self.queue_items.remove(index);
-        self.sync_items();
-        self.view.active(COMPONENT_TABLE);
+        self.sync_queue();
     }
 
     pub fn empty_queue(&mut self) {
         self.queue_items.clear();
-        self.sync_items();
+        self.sync_queue();
         self.view.active(COMPONENT_TREEVIEW);
     }
 
     pub fn save_queue(&mut self) -> Result<()> {
-        let mut path = self.get_app_config_path()?;
+        let mut path = get_app_config_path()?;
         path.push("queue.log");
 
         let mut file = File::create(path.as_path())?;
-        for i in self.queue_items.iter() {
+        for i in self.queue_items.iter().rev() {
             if let Some(f) = &i.file {
                 writeln!(&mut file, "{}", f)?;
             }
@@ -125,17 +119,8 @@ impl MainActivity {
         Ok(())
     }
 
-    pub fn get_app_config_path(&mut self) -> Result<PathBuf> {
-        let mut path = dirs_next::home_dir()
-            .map(|h| h.join(".config"))
-            .ok_or_else(|| anyhow!("failed to find os config dir."))?;
-
-        path.push("termusic");
-        fs::create_dir_all(&path)?;
-        Ok(path)
-    }
     pub fn load_queue(&mut self) -> Result<()> {
-        let mut path = self.get_app_config_path()?;
+        let mut path = get_app_config_path()?;
         path.push("queue.log");
 
         let file = match File::open(path.as_path()) {
@@ -151,21 +136,23 @@ impl MainActivity {
             .lines()
             .map(|line| line.unwrap_or_else(|_| "Error".to_string()))
             .collect();
-        for line in lines.iter().rev() {
+
+        for line in lines.iter() {
             match Song::from_str(line) {
-                Ok(s) => self.add_queue(s),
+                Ok(s) => self.queue_items.insert(0, s),
                 Err(e) => bail!("song add to queue error: {}", e),
             };
         }
 
+        self.sync_queue();
+
         Ok(())
     }
 
     pub fn shuffle(&mut self) {
         let mut rng = thread_rng();
         self.queue_items.shuffle(&mut rng);
-        self.sync_items();
-        self.view.active(COMPONENT_TABLE);
+        self.sync_queue();
     }
 
     pub fn update_item_delete(&mut self) {
@@ -178,15 +165,13 @@ impl MainActivity {
             }
         });
 
-        self.sync_items();
+        self.sync_queue();
         self.view.active(COMPONENT_TREEVIEW);
     }
     pub fn update_title(&self) -> String {
         let mut duration = Duration::from_secs(0);
         for v in self.queue_items.iter() {
-            if let Some(d) = v.duration {
-                duration += d;
-            }
+            duration += v.duration;
         }
         format!(
             "─ Queue ───┤ Total {} songs | {} ├─",
