@@ -22,30 +22,28 @@
  * SOFTWARE.
  */
 // locals
-use super::{
-    MainActivity, Status, COMPONENT_CONFIRMATION_INPUT, COMPONENT_CONFIRMATION_RADIO,
-    COMPONENT_INPUT_URL, COMPONENT_PARAGRAPH_LYRIC, COMPONENT_PROGRESS, COMPONENT_TABLE,
-    COMPONENT_TEXT_ERROR, COMPONENT_TEXT_HELP, COMPONENT_TREEVIEW,
-};
-use crate::song::Song;
-use crate::ui::keymap::*;
 use std::str::FromStr;
 // ext
-use super::{StatusLine, TransferState};
-use crate::player::AudioPlayer;
-use crate::songtag::lrc::Lyric;
+use super::{
+    youtube_options::YoutubeSearchState, ExitReason, MainActivity, MessageState, Status,
+    StatusLine, TransferState, COMPONENT_CONFIRMATION_INPUT, COMPONENT_CONFIRMATION_RADIO,
+    COMPONENT_INPUT_URL, COMPONENT_LABEL_HELP, COMPONENT_PARAGRAPH_LYRIC, COMPONENT_PROGRESS,
+    COMPONENT_TABLE_QUEUE, COMPONENT_TABLE_YOUTUBE, COMPONENT_TEXT_ERROR, COMPONENT_TEXT_HELP,
+    COMPONENT_TREEVIEW,
+};
+use crate::{player::AudioPlayer, song::Song, songtag::lrc::Lyric, ui::keymap::*};
 use humantime::format_duration;
 use std::path::{Path, PathBuf};
-use std::thread;
-use std::thread::sleep;
+use std::thread::{self, sleep};
 use std::time::Duration;
-use tui_realm_stdlib::{label, paragraph, progress_bar};
+use tui_realm_stdlib::{LabelPropsBuilder, ParagraphPropsBuilder, ProgressBarPropsBuilder};
 use tui_realm_treeview::TreeViewPropsBuilder;
-use tuirealm::event::{Event, KeyCode, KeyEvent, KeyModifiers};
-use tuirealm::props::TextSpan;
-use tuirealm::tui::layout::Alignment;
-use tuirealm::PropsBuilder;
-use tuirealm::{Msg, Payload, Value};
+use tuirealm::{
+    event::{Event, KeyCode, KeyEvent, KeyModifiers},
+    props::TextSpan,
+    tui::layout::Alignment,
+    Msg, Payload, PropsBuilder, Value,
+};
 
 impl MainActivity {
     /// ### update
@@ -58,10 +56,10 @@ impl MainActivity {
             None => None, // Exit after None
             Some(msg) => match msg {
                 (COMPONENT_TREEVIEW,key) if key== &MSG_KEY_TAB => {
-                    self.view.active(COMPONENT_TABLE);
+                    self.view.active(COMPONENT_TABLE_QUEUE);
                     None
                 }
-                (COMPONENT_TABLE, key) if key== &MSG_KEY_TAB => {
+                (COMPONENT_TABLE_QUEUE, key) if key== &MSG_KEY_TAB => {
                     self.view.active(COMPONENT_TREEVIEW);
                     None
                 }
@@ -94,9 +92,7 @@ impl MainActivity {
                 }
                 (COMPONENT_TREEVIEW, key) if key== &MSG_KEY_BACKSPACE => {
                     // Update tree
-                    match self.upper_dir() {
-                        None => None,
-                        Some(p) => {
+                    if let Some(p)  = self.upper_dir() {
                             let p: PathBuf = p.to_path_buf();
                             self.scan_dir(p.as_path());
                             // Update
@@ -108,14 +104,14 @@ impl MainActivity {
                                 let msg = self.view.update(COMPONENT_TREEVIEW, props);
                                 self.update(msg);
                             }
-                            None
-                        }
                     }
+                    None
                 }
                 // seek
                 (_, key) if key== &MSG_KEY_CHAR_F => match self.player.seek(5) {
                     Ok(_) =>{
                         self.time_pos += 5;
+                        self.update_progress();
                         None
                     },
                     Err(_) => {
@@ -137,15 +133,15 @@ impl MainActivity {
                 // adjust lyric delay 
                 (_, key) if key== &MSG_KEY_CHAR_CAPITAL_F => {
                    if let Some(song) = self.current_song.as_mut(){
-                   if let Some(lyric) = song.parsed_lyric.as_mut() {
-                       lyric.adjust_offset(self.time_pos,1000);
+                       if let Some(lyric) = song.parsed_lyric.as_mut() {
+                           lyric.adjust_offset(self.time_pos,1000);
                            if let Some(text) = lyric.as_lrc_text() {
-                                song.set_lyric(&text,"Adjusted");
+                               song.set_lyric(&text,"Adjusted");
                            }
-                           if let Err(e) = song.save() {
+                           if let Err(e) = song.save_tag() {
                                self.mount_error(e.to_string().as_ref());
                            };
-                   }
+                       }
                 }
                    None
                 }
@@ -153,18 +149,16 @@ impl MainActivity {
                 // adjust lyric delay 
                 (_, key) if key== &MSG_KEY_CHAR_CAPITAL_B =>  {
                     if let Some(song) = self.current_song.as_mut() {
-                   if let Some(lyric) = song.parsed_lyric.as_mut() {
-                       lyric.adjust_offset(self.time_pos,-1000);
-                            if let Some(text) = lyric.as_lrc_text() {
+                        if let Some(lyric) = song.parsed_lyric.as_mut() {
+                           lyric.adjust_offset(self.time_pos,-1000);
+                           if let Some(text) = lyric.as_lrc_text() {
                                    song.set_lyric(&text,"Adjusted");
                            }
-                           if let Err(e) = song.save() {
+                           if let Err(e) = song.save_tag() {
                                self.mount_error(e.to_string().as_ref());
                            };
-
-                   }
+                        }
                 }
-
                     None
                 },
 
@@ -177,7 +171,7 @@ impl MainActivity {
                     self.view.on(event);
                     None
                 }
-                (COMPONENT_TABLE, key) if key== &MSG_KEY_CHAR_G => {
+                (COMPONENT_TABLE_QUEUE, key) if key== &MSG_KEY_CHAR_G => {
                     let event: Event = Event::Key(KeyEvent {
                         code: KeyCode::Home,
                         modifiers: KeyModifiers::NONE,
@@ -186,7 +180,7 @@ impl MainActivity {
                     None
                 }
 
-                (COMPONENT_TABLE,key) if key==  &MSG_KEY_CHAR_CAPITAL_G => {
+                (COMPONENT_TABLE_QUEUE,key) if key==  &MSG_KEY_CHAR_CAPITAL_G => {
                     let event: Event = Event::Key(KeyEvent {
                         code: KeyCode::End,
                         modifiers: KeyModifiers::NONE,
@@ -235,7 +229,6 @@ impl MainActivity {
                     if let Some(Payload::One(Value::Str(node_id))) = self.view.get_state(COMPONENT_TREEVIEW) {
                             let p: &Path = Path::new(node_id.as_str());
                             if p.is_dir() {
-                                // let p = p.to_string_lossy();
                                 let new_items = Self::dir_children(p);
                                 for i in new_items.iter().rev() {
                                     match Song::from_str(i) {
@@ -248,25 +241,22 @@ impl MainActivity {
                             None
                 }
 
-                (COMPONENT_TABLE,key) if key==  &MSG_KEY_CHAR_L => {
-                    match self.view.get_state(COMPONENT_TABLE) {
-                        Some(Payload::One(Value::Usize(index))) => {
-                            self.time_pos = 0;
-                            if let Some(song) = self.queue_items.get(index) {
-                                if let Some(file) = &song.file {
-                                    self.player.queue_and_play(file);
-                                }
-                                self.current_song = Some(song.to_owned());
+                (COMPONENT_TABLE_QUEUE,key) if key==  &MSG_KEY_CHAR_L => {
+                    if let Some(Payload::One(Value::Usize(index))) = self.view.get_state(COMPONENT_TABLE_QUEUE) {
+                        self.time_pos = 0;
+                        if let Some(song) = self.queue_items.get(index) {
+                            if let Some(file) = &song.file {
+                                self.player.queue_and_play(file);
                             }
-                            self.update_photo();
-                            None
+                            self.current_song = Some(song.to_owned());
                         }
-                        _ => None,
+                        self.update_photo();
                     }
+                    None
                 }
 
-                (COMPONENT_TABLE,key) if key==  &MSG_KEY_CHAR_D => {
-                    match self.view.get_state(COMPONENT_TABLE) {
+                (COMPONENT_TABLE_QUEUE,key) if key==  &MSG_KEY_CHAR_D => {
+                    match self.view.get_state(COMPONENT_TABLE_QUEUE) {
                         Some(Payload::One(Value::Usize(index))) => {
                             self.delete_item(index);
                             None
@@ -275,7 +265,7 @@ impl MainActivity {
                     }
                 }
 
-                (COMPONENT_TABLE,key) if key==  &MSG_KEY_CHAR_CAPITAL_D => {
+                (COMPONENT_TABLE_QUEUE,key) if key==  &MSG_KEY_CHAR_CAPITAL_D => {
                     self.empty_queue();
                     None
                 }
@@ -297,7 +287,7 @@ impl MainActivity {
                     None
                 }
                 // shuffle
-                (COMPONENT_TABLE,key) if key==  &MSG_KEY_CHAR_S => {
+                (COMPONENT_TABLE_QUEUE,key) if key==  &MSG_KEY_CHAR_S => {
                     self.shuffle();
                     None
                 }
@@ -337,23 +327,23 @@ impl MainActivity {
                     None
                 }
 
-                (super::COMPONENT_SCROLLTABLE_YOUTUBE,key) if key== &MSG_KEY_TAB => {
+                (COMPONENT_TABLE_YOUTUBE,key) if key== &MSG_KEY_TAB => {
                     self.youtube_options_next_page();
                     None
                 }
 
-                (super::COMPONENT_SCROLLTABLE_YOUTUBE,key) if key== &MSG_KEY_SHIFT_TAB => {
+                (COMPONENT_TABLE_YOUTUBE,key) if key== &MSG_KEY_SHIFT_TAB => {
                     self.youtube_options_prev_page();
                     None
                 }
 
-                (super::COMPONENT_SCROLLTABLE_YOUTUBE,key) if (key== &MSG_KEY_ESC) | (key == &MSG_KEY_CHAR_CAPITAL_Q)  => {
+                (COMPONENT_TABLE_YOUTUBE,key) if (key== &MSG_KEY_ESC) | (key == &MSG_KEY_CHAR_CAPITAL_Q)  => {
                     self.umount_youtube_options();
                     None
                 }
 
-                (super::COMPONENT_SCROLLTABLE_YOUTUBE,key) if key== &MSG_KEY_ENTER => {
-                    if let Some(Payload::One(Value::Usize(index))) = self.view.get_state(super::COMPONENT_SCROLLTABLE_YOUTUBE) {
+                (COMPONENT_TABLE_YOUTUBE,key) if key== &MSG_KEY_ENTER => {
+                    if let Some(Payload::One(Value::Usize(index))) = self.view.get_state(COMPONENT_TABLE_YOUTUBE) {
                         // download from search result here
                         if let Err(e) = self.youtube_options_download(index) {
                             self.mount_error(format!("download song error: {}",e).as_str());
@@ -420,7 +410,7 @@ impl MainActivity {
 
                 // switch lyrics
                 (_,key) if key == &MSG_KEY_CHAR_CAPITAL_T => {
-                    if let Some(mut song) = self.current_song.clone() {
+                    if let Some(mut song) = self.current_song.to_owned() {
                         if song.lyric_frames.is_empty() {
                             return None
                         }
@@ -432,12 +422,12 @@ impl MainActivity {
                             if let Ok(parsed_lyric) = Lyric::from_str(&f.text) {
                                 let tx = self.sender_message.clone();
                                 song.parsed_lyric = Some(parsed_lyric);
-                                let lang_ext = f.description.clone();
+                                let lang_ext = f.description.to_owned();
                                 self.current_song = Some(song);
                                 thread::spawn(move || {
-                                    let _ = tx.send(super::MessageState::Show(("Lyric switch successful".to_string(),format!("{} lyric is showing",lang_ext))));
+                                    let _ = tx.send(MessageState::Show(("Lyric switch successful".to_string(),format!("{} lyric is showing",lang_ext))));
                                     sleep(Duration::from_secs(5));
-                                    let _ = tx.send(super::MessageState::Hide);
+                                    let _ = tx.send(MessageState::Hide);
                                 });
                             }
                         }
@@ -462,7 +452,7 @@ impl MainActivity {
                     None
                 }
 
-                (_,key) if key==  &MSG_KEY_QUESTION_MARK => {
+                (_,key) if key==  &MSG_KEY_CTRL_H => {
                     // Show help
                     self.mount_help();
                     None
@@ -485,13 +475,13 @@ impl MainActivity {
 
                 // Refresh playlist
                 (_,key) if key==  &MSG_KEY_CHAR_R => {
-                    self.refresh_playlist(None);
+                    self.sync_playlist(None);
                     None
                 }
 
                 (_,key) if (key==  &MSG_KEY_ESC) | (key == &MSG_KEY_CHAR_CAPITAL_Q) => {
                     // Quit on esc
-                    self.exit_reason = Some(super::ExitReason::Quit);
+                    self.exit_reason = Some(ExitReason::Quit);
                     None
                 }
                 _ => None,
@@ -499,6 +489,34 @@ impl MainActivity {
         }
     }
 
+    pub fn update_progress_title(&mut self) {
+        if let Some(song) = &self.current_song {
+            let artist = song.artist().unwrap_or("Unknown Artist");
+            let title = song.title().unwrap_or("Unknown Title");
+            if let Some(props) = self.view.get_props(COMPONENT_PROGRESS) {
+                let props = ProgressBarPropsBuilder::from(props)
+                    // .with_progress(new_prog)
+                    .with_title(
+                        format!("Playing: {} - {}", artist, title),
+                        Alignment::Center,
+                    )
+                    .build();
+                let msg = self.view.update(COMPONENT_PROGRESS, props);
+                // self.redraw = true;
+                self.update(msg);
+            }
+        };
+    }
+
+    pub fn update_duration(&mut self) {
+        let (_new_prog, _time_pos, duration) = self.player.get_progress();
+        if let Some(song) = &mut self.current_song {
+            let diff = song.duration.as_secs() as i64 - duration;
+            if diff.abs() > 1 {
+                song.update_duration();
+            }
+        }
+    }
     pub fn update_progress(&mut self) {
         let (new_prog, time_pos, duration) = self.player.get_progress();
         if (new_prog, time_pos, duration) == (0.9, 0, 100) {
@@ -510,33 +528,25 @@ impl MainActivity {
             return;
         }
 
-        let song = match self.current_song.clone() {
-            Some(song) => song,
+        let song = match self.current_song.to_owned() {
+            Some(s) => s,
             None => return,
         };
 
-        let artist = song.artist().unwrap_or("Unknown Artist");
-        let title = song.title().unwrap_or("Unknown Title");
-
         if time_pos > self.time_pos || time_pos < 2 {
             self.time_pos = time_pos;
             if let Some(props) = self.view.get_props(COMPONENT_PROGRESS) {
-                let props = progress_bar::ProgressBarPropsBuilder::from(props)
+                let props = ProgressBarPropsBuilder::from(props)
                     .with_progress(new_prog)
-                    .with_title(
-                        format!("Playing: {} - {}", artist, title),
-                        Alignment::Center,
-                    )
                     .with_label(format!(
                         "{}     :     {} ",
                         format_duration(Duration::from_secs(time_pos as u64)),
                         format_duration(Duration::from_secs(duration as u64))
                     ))
                     .build();
-                self.view.update(COMPONENT_PROGRESS, props);
+                let msg = self.view.update(COMPONENT_PROGRESS, props);
                 self.redraw = true;
-                self.view();
-                self.redraw = false;
+                self.update(msg);
             }
         }
 
@@ -547,8 +557,7 @@ impl MainActivity {
 
         if song.lyric_frames.is_empty() {
             if let Some(props) = self.view.get_props(COMPONENT_PARAGRAPH_LYRIC) {
-                let props = paragraph::ParagraphPropsBuilder::from(props)
-                    .with_title("Lyrics", Alignment::Left)
+                let props = ParagraphPropsBuilder::from(props)
                     .with_texts(vec![TextSpan::new("No lyrics available.")])
                     .build();
                 self.view.update(COMPONENT_PARAGRAPH_LYRIC, props);
@@ -567,8 +576,7 @@ impl MainActivity {
         }
 
         if let Some(props) = self.view.get_props(COMPONENT_PARAGRAPH_LYRIC) {
-            let props = paragraph::ParagraphPropsBuilder::from(props)
-                .with_title("Lyrics", Alignment::Left)
+            let props = ParagraphPropsBuilder::from(props)
                 .with_texts(vec![TextSpan::new(line)])
                 .build();
             self.view.update(COMPONENT_PARAGRAPH_LYRIC, props);
@@ -633,11 +641,11 @@ impl MainActivity {
                     self.update_status_line(StatusLine::Success);
                 }
                 TransferState::Completed(Some(file)) => {
-                    self.refresh_playlist(Some(file.as_str()));
+                    self.sync_playlist(Some(file.as_str()));
                     self.update_status_line(StatusLine::Default);
                 }
                 TransferState::Completed(None) => {
-                    self.refresh_playlist(None);
+                    self.sync_playlist(None);
                     self.update_status_line(StatusLine::Default);
                 }
                 TransferState::ErrDownload => {
@@ -655,15 +663,15 @@ impl MainActivity {
     pub fn update_status_line(&mut self, s: StatusLine) {
         match s {
             StatusLine::Default => {
-                let text = "Press \"?\" for help.".to_string();
-                if let Some(props) = self.view.get_props(super::COMPONENT_LABEL_HELP) {
-                    let props = label::LabelPropsBuilder::from(props)
+                let text = "Press <CTRL+H> for help.".to_string();
+                if let Some(props) = self.view.get_props(COMPONENT_LABEL_HELP) {
+                    let props = LabelPropsBuilder::from(props)
                         .with_text(text)
                         .with_background(tuirealm::tui::style::Color::Reset)
                         .with_foreground(tuirealm::tui::style::Color::Cyan)
                         .build();
 
-                    let msg = self.view.update(super::COMPONENT_LABEL_HELP, props);
+                    let msg = self.view.update(COMPONENT_LABEL_HELP, props);
                     self.update(msg);
                     self.redraw = true;
                 }
@@ -671,14 +679,14 @@ impl MainActivity {
             StatusLine::Running => {
                 let text = " Downloading...".to_string();
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_LABEL_HELP) {
-                    let props = label::LabelPropsBuilder::from(props)
+                if let Some(props) = self.view.get_props(COMPONENT_LABEL_HELP) {
+                    let props = LabelPropsBuilder::from(props)
                         .with_text(text)
                         .with_foreground(tuirealm::tui::style::Color::White)
                         .with_background(tuirealm::tui::style::Color::Red)
                         .build();
 
-                    let msg = self.view.update(super::COMPONENT_LABEL_HELP, props);
+                    let msg = self.view.update(COMPONENT_LABEL_HELP, props);
                     self.update(msg);
                     self.redraw = true;
                 }
@@ -686,14 +694,14 @@ impl MainActivity {
             StatusLine::Success => {
                 let text = " Download Success!".to_string();
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_LABEL_HELP) {
-                    let props = label::LabelPropsBuilder::from(props)
+                if let Some(props) = self.view.get_props(COMPONENT_LABEL_HELP) {
+                    let props = LabelPropsBuilder::from(props)
                         .with_text(text)
-                        .with_foreground(tuirealm::tui::style::Color::White)
+                        .with_foreground(tuirealm::tui::style::Color::Black)
                         .with_background(tuirealm::tui::style::Color::Green)
                         .build();
 
-                    let msg = self.view.update(super::COMPONENT_LABEL_HELP, props);
+                    let msg = self.view.update(COMPONENT_LABEL_HELP, props);
                     self.update(msg);
                     self.redraw = true;
                 }
@@ -701,14 +709,14 @@ impl MainActivity {
             StatusLine::Error => {
                 let text = " Download Error!".to_string();
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_LABEL_HELP) {
-                    let props = label::LabelPropsBuilder::from(props)
+                if let Some(props) = self.view.get_props(COMPONENT_LABEL_HELP) {
+                    let props = LabelPropsBuilder::from(props)
                         .with_text(text)
                         .with_foreground(tuirealm::tui::style::Color::White)
                         .with_background(tuirealm::tui::style::Color::Red)
                         .build();
 
-                    let msg = self.view.update(super::COMPONENT_LABEL_HELP, props);
+                    let msg = self.view.update(COMPONENT_LABEL_HELP, props);
                     self.update(msg);
                     self.redraw = true;
                 }
@@ -720,19 +728,29 @@ impl MainActivity {
     pub fn update_message(&mut self) {
         if let Ok(message_state) = self.receiver_message.try_recv() {
             match message_state {
-                super::MessageState::Show((title, text)) => {
-                    // if let Some(props) = self.view.get_props(super::COMPONENT_TEXT_MESSAGE) {
-                    //     if props.visible {
-                    //         self.umount_message();
-                    //     }
-                    // }
-
+                MessageState::Show((title, text)) => {
                     self.mount_message(&title, &text);
                 }
-                super::MessageState::Hide => {
+                MessageState::Hide => {
                     self.umount_message();
                 }
             }
         }
     }
+
+    // update youtube search box
+    pub fn update_youtube_search(&mut self) {
+        if let Ok(youtube_search) = self.receiver_youtubesearch.try_recv() {
+            match youtube_search {
+                YoutubeSearchState::Success(y) => {
+                    self.youtube_options = y;
+                    self.sync_youtube_options();
+                    self.redraw = true;
+                }
+                YoutubeSearchState::Fail(e) => {
+                    self.mount_error(&e);
+                }
+            }
+        }
+    }
 }
