@@ -23,26 +23,35 @@
  */
 // Locals
 use super::{
-    MainActivity, COMPONENT_LABEL_HELP, COMPONENT_PARAGRAPH_LYRIC, COMPONENT_PROGRESS,
-    COMPONENT_TABLE, COMPONENT_TREEVIEW,
+    MainActivity, COMPONENT_CONFIRMATION_INPUT, COMPONENT_CONFIRMATION_RADIO, COMPONENT_INPUT_URL,
+    COMPONENT_LABEL_HELP, COMPONENT_PARAGRAPH_LYRIC, COMPONENT_PROGRESS, COMPONENT_TABLE_QUEUE,
+    COMPONENT_TABLE_YOUTUBE, COMPONENT_TEXT_ERROR, COMPONENT_TEXT_HELP, COMPONENT_TEXT_MESSAGE,
+    COMPONENT_TREEVIEW,
+};
+use crate::ui::{
+    components::msgbox::{MsgBox, MsgBoxPropsBuilder},
+    draw_area_in, draw_area_top_right,
 };
-use crate::ui::components::msgbox::{MsgBox, MsgBoxPropsBuilder};
-use crate::ui::components::table;
-use crate::ui::{draw_area_in, draw_area_top_right};
 // Ext
 use tui_realm_stdlib::{
-    input, label, paragraph, progress_bar, radio,
-    table::{Table, TablePropsBuilder},
+    Input, InputPropsBuilder, Label, LabelPropsBuilder, Paragraph, ParagraphPropsBuilder,
+    ProgressBar, ProgressBarPropsBuilder, Radio, RadioPropsBuilder, Table, TablePropsBuilder,
 };
 
-use tuirealm::props::borders::{BorderType, Borders};
-use tuirealm::props::{TableBuilder, TextSpan};
-use tuirealm::{PropsBuilder, View};
+use tuirealm::{
+    props::{
+        borders::{BorderType, Borders},
+        TableBuilder, TextSpan,
+    },
+    tui::{
+        layout::{Alignment, Constraint, Direction, Layout},
+        style::Color,
+        widgets::Clear,
+    },
+    PropsBuilder, View,
+};
 // tui
 use tui_realm_treeview::{TreeView, TreeViewPropsBuilder};
-use tuirealm::tui::layout::{Alignment, Constraint, Direction, Layout};
-use tuirealm::tui::style::Color;
-use tuirealm::tui::widgets::Clear;
 
 impl MainActivity {
     // -- view
@@ -56,8 +65,8 @@ impl MainActivity {
         // Let's mount the component we need
         self.view.mount(
             COMPONENT_PROGRESS,
-            Box::new(progress_bar::ProgressBar::new(
-                progress_bar::ProgressBarPropsBuilder::default()
+            Box::new(ProgressBar::new(
+                ProgressBarPropsBuilder::default()
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::LightMagenta)
                     .with_progbar_color(Color::LightYellow)
                     .with_title("Playing", Alignment::Center)
@@ -69,17 +78,17 @@ impl MainActivity {
         );
         self.view.mount(
             COMPONENT_LABEL_HELP,
-            Box::new(label::Label::new(
-                label::LabelPropsBuilder::default()
+            Box::new(Label::new(
+                LabelPropsBuilder::default()
                     .with_foreground(Color::Cyan)
-                    .with_text(String::from("Press \"?\" for help."))
+                    .with_text(String::from("Press <CTRL+H> for help."))
                     .build(),
             )),
         );
         self.view.mount(
             COMPONENT_PARAGRAPH_LYRIC,
-            Box::new(paragraph::Paragraph::new(
-                paragraph::ParagraphPropsBuilder::default()
+            Box::new(Paragraph::new(
+                ParagraphPropsBuilder::default()
                     .with_foreground(Color::Cyan)
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::Green)
                     .with_title("Lyrics", Alignment::Left)
@@ -92,16 +101,15 @@ impl MainActivity {
 
         // Scrolltable
         self.view.mount(
-            COMPONENT_TABLE,
-            Box::new(table::Table::new(
-                table::TablePropsBuilder::default()
+            COMPONENT_TABLE_QUEUE,
+            Box::new(Table::new(
+                TablePropsBuilder::default()
                     .with_background(Color::Black)
                     .with_highlighted_str(Some("🚀"))
                     .with_highlighted_color(Color::LightBlue)
                     .with_max_scroll_step(4)
-                    .with_borders(Borders::ALL, BorderType::Rounded, Color::Blue)
+                    .with_borders(Borders::ALL, BorderType::Thick, Color::Blue)
                     .scrollable(true)
-                    // .with_title(" Duration ┼─── Artist ───┼────── Title ───────┼────── Album ──────┼────── Queue─")
                     .with_title("Queue", Alignment::Left)
                     .with_header(&["Duration", "Artist", "Title", "Album"])
                     .with_widths(&[10, 20, 25, 45])
@@ -165,74 +173,71 @@ impl MainActivity {
 
                 self.view.render(COMPONENT_TREEVIEW, f, chunks_left[0]);
                 self.view.render(COMPONENT_LABEL_HELP, f, chunks_main[1]);
-                self.view.render(COMPONENT_TABLE, f, chunks_right[0]);
+                self.view.render(COMPONENT_TABLE_QUEUE, f, chunks_right[0]);
                 self.view.render(COMPONENT_PROGRESS, f, chunks_right[1]);
                 self.view
                     .render(COMPONENT_PARAGRAPH_LYRIC, f, chunks_right[2]);
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_TEXT_HELP) {
+                if let Some(props) = self.view.get_props(COMPONENT_TEXT_HELP) {
                     if props.visible {
                         // make popup
                         let popup = draw_area_in(f.size(), 50, 70);
                         f.render_widget(Clear, popup);
-                        self.view.render(super::COMPONENT_TEXT_HELP, f, popup);
+                        self.view.render(COMPONENT_TEXT_HELP, f, popup);
                     }
                 }
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_INPUT_URL) {
+                if let Some(props) = self.view.get_props(COMPONENT_INPUT_URL) {
                     if props.visible {
                         // make popup
                         let popup = draw_area_in(f.size(), 50, 10);
                         f.render_widget(Clear, popup);
-                        self.view.render(super::COMPONENT_INPUT_URL, f, popup);
+                        self.view.render(COMPONENT_INPUT_URL, f, popup);
                     }
                 }
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_TEXT_ERROR) {
+                if let Some(props) = self.view.get_props(COMPONENT_TEXT_ERROR) {
                     if props.visible {
                         let popup = draw_area_in(f.size(), 50, 10);
                         f.render_widget(Clear, popup);
                         // make popup
-                        self.view.render(super::COMPONENT_TEXT_ERROR, f, popup);
+                        self.view.render(COMPONENT_TEXT_ERROR, f, popup);
                     }
                 }
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_TEXT_MESSAGE) {
+                if let Some(props) = self.view.get_props(COMPONENT_TEXT_MESSAGE) {
                     if props.visible {
                         let popup = draw_area_top_right(f.size(), 32, 15);
                         f.render_widget(Clear, popup);
                         // make popup
-                        self.view.render(super::COMPONENT_TEXT_MESSAGE, f, popup);
+                        self.view.render(COMPONENT_TEXT_MESSAGE, f, popup);
                     }
                 }
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_CONFIRMATION_RADIO) {
+                if let Some(props) = self.view.get_props(COMPONENT_CONFIRMATION_RADIO) {
                     if props.visible {
                         let popup = draw_area_in(f.size(), 20, 10);
                         f.render_widget(Clear, popup);
                         // make popup
-                        self.view
-                            .render(super::COMPONENT_CONFIRMATION_RADIO, f, popup);
+                        self.view.render(COMPONENT_CONFIRMATION_RADIO, f, popup);
                     }
                 }
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_CONFIRMATION_INPUT) {
+                if let Some(props) = self.view.get_props(COMPONENT_CONFIRMATION_INPUT) {
                     if props.visible {
                         let popup = draw_area_in(f.size(), 20, 10);
                         f.render_widget(Clear, popup);
                         // make popup
-                        self.view
-                            .render(super::COMPONENT_CONFIRMATION_INPUT, f, popup);
+                        self.view.render(COMPONENT_CONFIRMATION_INPUT, f, popup);
                     }
                 }
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_SCROLLTABLE_YOUTUBE) {
+                if let Some(props) = self.view.get_props(COMPONENT_TABLE_YOUTUBE) {
                     if props.visible {
                         let popup = draw_area_in(f.size(), 66, 60);
                         f.render_widget(Clear, popup);
                         // make popup
-                        self.view
-                            .render(super::COMPONENT_SCROLLTABLE_YOUTUBE, f, popup);
+                        self.view.render(COMPONENT_TABLE_YOUTUBE, f, popup);
                     }
                 }
             });
@@ -248,7 +253,7 @@ impl MainActivity {
     pub(super) fn mount_error(&mut self, text: &str) {
         // Mount
         self.view.mount(
-            super::COMPONENT_TEXT_ERROR,
+            COMPONENT_TEXT_ERROR,
             Box::new(MsgBox::new(
                 MsgBoxPropsBuilder::default()
                     .with_foreground(Color::Red)
@@ -259,14 +264,14 @@ impl MainActivity {
             )),
         );
         // Give focus to error
-        self.view.active(super::COMPONENT_TEXT_ERROR);
+        self.view.active(COMPONENT_TEXT_ERROR);
     }
 
     /// ### umount_error
     ///
     /// Umount error message
     pub(super) fn umount_error(&mut self) {
-        self.view.umount(super::COMPONENT_TEXT_ERROR);
+        self.view.umount(COMPONENT_TEXT_ERROR);
     }
     // ### mount_message
     //
@@ -274,7 +279,7 @@ impl MainActivity {
     pub(super) fn mount_message(&mut self, title: &str, text: &str) {
         // Mount
         self.view.mount(
-            super::COMPONENT_TEXT_MESSAGE,
+            COMPONENT_TEXT_MESSAGE,
             Box::new(MsgBox::new(
                 MsgBoxPropsBuilder::default()
                     .with_foreground(Color::Green)
@@ -285,14 +290,14 @@ impl MainActivity {
             )),
         );
         // Give focus to error
-        self.view.active(super::COMPONENT_TEXT_MESSAGE);
+        self.view.active(COMPONENT_TEXT_MESSAGE);
     }
 
     /// ### umount_message
     ///
     /// Umount error message
     pub(super) fn umount_message(&mut self) {
-        self.view.umount(super::COMPONENT_TEXT_MESSAGE);
+        self.view.umount(COMPONENT_TEXT_MESSAGE);
     }
 
     /// ### mount_del_ssh_key
@@ -300,9 +305,9 @@ impl MainActivity {
     /// Mount delete ssh key component
     pub(super) fn mount_confirmation_radio(&mut self) {
         self.view.mount(
-            super::COMPONENT_CONFIRMATION_RADIO,
-            Box::new(radio::Radio::new(
-                radio::RadioPropsBuilder::default()
+            COMPONENT_CONFIRMATION_RADIO,
+            Box::new(Radio::new(
+                RadioPropsBuilder::default()
                     .with_color(Color::LightRed)
                     .with_inverted_color(Color::Black)
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::LightRed)
@@ -313,34 +318,34 @@ impl MainActivity {
             )),
         );
         // Active
-        self.view.active(super::COMPONENT_CONFIRMATION_RADIO);
+        self.view.active(COMPONENT_CONFIRMATION_RADIO);
     }
 
     /// ### umount_del_ssh_key
     ///
     /// Umount delete ssh key
     pub(super) fn umount_confirmation_radio(&mut self) {
-        self.view.umount(super::COMPONENT_CONFIRMATION_RADIO);
+        self.view.umount(COMPONENT_CONFIRMATION_RADIO);
     }
 
     pub(super) fn mount_confirmation_input(&mut self) {
         self.view.mount(
-            super::COMPONENT_CONFIRMATION_INPUT,
-            Box::new(input::Input::new(
-                input::InputPropsBuilder::default()
+            COMPONENT_CONFIRMATION_INPUT,
+            Box::new(Input::new(
+                InputPropsBuilder::default()
                     .with_label(String::from("Type DELETE to confirm:"), Alignment::Left)
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::Green)
                     .build(),
             )),
         );
-        self.view.active(super::COMPONENT_CONFIRMATION_INPUT);
+        self.view.active(COMPONENT_CONFIRMATION_INPUT);
     }
 
     /// ### umount_new_ssh_key
     ///
     /// Umount new ssh key prompt
     pub(super) fn umount_confirmation_input(&mut self) {
-        self.view.umount(super::COMPONENT_CONFIRMATION_INPUT);
+        self.view.umount(COMPONENT_CONFIRMATION_INPUT);
     }
 
     /// ### mount_new_ssh_key
@@ -348,22 +353,22 @@ impl MainActivity {
     /// Mount new ssh key prompt
     pub(super) fn mount_youtube_url(&mut self) {
         self.view.mount(
-            super::COMPONENT_INPUT_URL,
-            Box::new(input::Input::new(
-                input::InputPropsBuilder::default()
+            COMPONENT_INPUT_URL,
+            Box::new(Input::new(
+                InputPropsBuilder::default()
                     .with_label(String::from("Download url or search:"), Alignment::Left)
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::Green)
                     .build(),
             )),
         );
-        self.view.active(super::COMPONENT_INPUT_URL);
+        self.view.active(COMPONENT_INPUT_URL);
     }
 
     /// ### umount_new_ssh_key
     ///
     /// Umount new ssh key prompt
     pub(super) fn umount_youtube_url(&mut self) {
-        self.view.umount(super::COMPONENT_INPUT_URL);
+        self.view.umount(COMPONENT_INPUT_URL);
     }
 
     // /// ### mount_help
@@ -371,7 +376,7 @@ impl MainActivity {
     // /// Mount help
     pub(super) fn mount_help(&mut self) {
         self.view.mount(
-            super::COMPONENT_TEXT_HELP,
+            COMPONENT_TEXT_HELP,
             Box::new(Table::new(
                 TablePropsBuilder::default()
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::Green)
@@ -440,14 +445,14 @@ impl MainActivity {
             )),
         );
         // Active help
-        self.view.active(super::COMPONENT_TEXT_HELP);
+        self.view.active(COMPONENT_TEXT_HELP);
     }
 
     /// ### umount_help
     ///
     /// Umount help
     pub(super) fn umount_help(&mut self) {
-        self.view.umount(super::COMPONENT_TEXT_HELP);
+        self.view.umount(COMPONENT_TEXT_HELP);
     }
 
     /// ### mount_youtube_options
@@ -455,9 +460,9 @@ impl MainActivity {
     /// Mount youtube options
     pub(super) fn mount_youtube_options(&mut self) {
         self.view.mount(
-            super::COMPONENT_SCROLLTABLE_YOUTUBE,
-            Box::new(table::Table::new(
-                table::TablePropsBuilder::default()
+            COMPONENT_TABLE_YOUTUBE,
+            Box::new(Table::new(
+                TablePropsBuilder::default()
                     .with_background(Color::Black)
                     .with_highlighted_str(Some("🚀"))
                     .with_highlighted_color(Color::LightBlue)
@@ -465,23 +470,25 @@ impl MainActivity {
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::Blue)
                     .with_title("Tab/Shift+Tab for next and previous page", Alignment::Left)
                     .scrollable(true)
+                    .with_widths(&[20, 80])
                     .with_table(
                         TableBuilder::default()
+                            .add_col(TextSpan::from("Empty result."))
                             .add_col(TextSpan::from(
-                                "empty queue.Probably the invidious server is down.",
+                                "Wait 10 seconds but no results, means all servers are down.",
                             ))
                             .build(),
                     )
                     .build(),
             )),
         );
-        self.view.active(super::COMPONENT_SCROLLTABLE_YOUTUBE);
+        self.view.active(COMPONENT_TABLE_YOUTUBE);
     }
 
     /// ### umount_youtube_options
     ///
     /// Umount youtube options
     pub(super) fn umount_youtube_options(&mut self) {
-        self.view.umount(super::COMPONENT_SCROLLTABLE_YOUTUBE);
+        self.view.umount(COMPONENT_TABLE_YOUTUBE);
     }
 }
