@@ -21,21 +21,21 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
-use super::{MainActivity, COMPONENT_TREEVIEW};
+use super::{MainActivity, TransferState, COMPONENT_TABLE_YOUTUBE, COMPONENT_TREEVIEW};
 use crate::invidious::{InvidiousInstance, YoutubeVideo};
-use crate::ui::components::table;
 use anyhow::{anyhow, bail, Result};
 use humantime::format_duration;
 use id3::frame::Lyrics;
 use lazy_static::lazy_static;
 use regex::Regex;
 use std::path::{Path, PathBuf};
-use std::thread;
-use std::thread::sleep;
+use std::thread::{self, sleep};
 use std::time::Duration;
-use tuirealm::props::{TableBuilder, TextSpan};
-use tuirealm::{Payload, PropsBuilder, Value};
-use unicode_truncate::{Alignment, UnicodeTruncateStr};
+use tui_realm_stdlib::TablePropsBuilder;
+use tuirealm::{
+    props::{TableBuilder, TextSpan},
+    Payload, PropsBuilder, Value,
+};
 use ytd_rs::{Arg, ResultType, YoutubeDL};
 
 lazy_static! {
@@ -46,16 +46,19 @@ lazy_static! {
 pub struct YoutubeOptions {
     items: Vec<YoutubeVideo>,
     page: u32,
-    search_word: String,
     invidious_instance: InvidiousInstance,
 }
 
+pub enum YoutubeSearchState {
+    Success(YoutubeOptions),
+    Fail(String),
+}
+
 impl YoutubeOptions {
     pub fn new() -> Self {
         Self {
             items: Vec::new(),
             page: 1,
-            search_word: "".to_string(),
             invidious_instance: crate::invidious::InvidiousInstance::default(),
         }
     }
@@ -66,26 +69,10 @@ impl YoutubeOptions {
         Err(anyhow!("index not found"))
     }
 
-    pub fn search(&mut self, keyword: &str) -> Result<()> {
-        self.search_word = keyword.to_string();
-        match crate::invidious::InvidiousInstance::new(keyword) {
-            Ok((instance, result)) => {
-                self.invidious_instance = instance;
-                self.items = result;
-                self.page = 1;
-                Ok(())
-            }
-            Err(e) => Err(e),
-        }
-    }
-
     pub fn prev_page(&mut self) -> Result<()> {
         if self.page > 1 {
             self.page -= 1;
-            match self
-                .invidious_instance
-                .get_search_query(self.search_word.as_str(), self.page)
-            {
+            match self.invidious_instance.get_search_query(self.page) {
                 Ok(y) => {
                     self.items = y;
                     Ok(())
@@ -99,10 +86,7 @@ impl YoutubeOptions {
 
     pub fn next_page(&mut self) -> Result<()> {
         self.page += 1;
-        match self
-            .invidious_instance
-            .get_search_query(self.search_word.as_str(), self.page)
-        {
+        match self.invidious_instance.get_search_query(self.page) {
             Ok(y) => {
                 self.items = y;
                 Ok(())
@@ -130,10 +114,23 @@ impl MainActivity {
     }
 
     pub fn youtube_options_search(&mut self, keyword: &str) {
-        match self.youtube_options.search(keyword) {
-            Ok(()) => self.sync_youtube_options(),
-            Err(e) => self.mount_error(format!("search error: {}", e).as_str()),
-        }
+        let search_word = keyword.to_string();
+        let tx = self.sender_youtubesearch.clone();
+        thread::spawn(
+            move || match crate::invidious::InvidiousInstance::new(&search_word) {
+                Ok((instance, result)) => {
+                    let youtube_options = YoutubeOptions {
+                        items: result,
+                        page: 1,
+                        invidious_instance: instance,
+                    };
+                    let _ = tx.send(YoutubeSearchState::Success(youtube_options));
+                }
+                Err(e) => {
+                    let _ = tx.send(YoutubeSearchState::Fail(e.to_string()));
+                }
+            },
+        );
     }
 
     pub fn youtube_options_prev_page(&mut self) {
@@ -150,23 +147,19 @@ impl MainActivity {
     }
     pub fn sync_youtube_options(&mut self) {
         if self.youtube_options.items.is_empty() {
-            if let Some(props) = self.view.get_props(super::COMPONENT_SCROLLTABLE_YOUTUBE) {
-                if let Some(domain) = &self.youtube_options.invidious_instance.domain {
-                    let props = table::TablePropsBuilder::from(props)
-                        .with_table(
-                            TableBuilder::default()
-                                .add_col(TextSpan::from(format!(
-                                    "Empty result.Probably {} is down.",
-                                    domain
-                                )))
-                                .build(),
-                        )
-                        .build();
-                    let msg = self
-                        .view
-                        .update(super::COMPONENT_SCROLLTABLE_YOUTUBE, props);
-                    self.update(msg);
-                }
+            if let Some(props) = self.view.get_props(COMPONENT_TABLE_YOUTUBE) {
+                let props = TablePropsBuilder::from(props)
+                    .with_table(
+                        TableBuilder::default()
+                            .add_col(TextSpan::from("Empty result."))
+                            .add_col(TextSpan::from(
+                                "Wait 10 seconds but no results, means all servers are down.",
+                            ))
+                            .build(),
+                    )
+                    .build();
+                let msg = self.view.update(COMPONENT_TABLE_YOUTUBE, props);
+                self.update(msg);
             }
             return;
         }
@@ -176,36 +169,33 @@ impl MainActivity {
             if idx > 0 {
                 table.add_row();
             }
-            let duration = record.length_seconds;
-            let duration_string = format!("{}", format_duration(Duration::from_secs(duration)));
-            let duration_truncated = duration_string.unicode_pad(6, Alignment::Left, true);
+            let mut duration =
+                format_duration(Duration::from_secs(record.length_seconds)).to_string();
+            duration.truncate(10);
+            let duration_string = format!("[{:^10}]", duration);
 
             let title = record.title.as_str();
 
             table
-                .add_col(TextSpan::new(
-                    format!("[{}] ", duration_truncated,).as_str(),
-                ))
+                .add_col(TextSpan::new(duration_string))
                 .add_col(TextSpan::new(title).bold());
         }
         let table = table.build();
 
-        if let Some(props) = self.view.get_props(super::COMPONENT_SCROLLTABLE_YOUTUBE) {
+        if let Some(props) = self.view.get_props(COMPONENT_TABLE_YOUTUBE) {
             if let Some(domain) = &self.youtube_options.invidious_instance.domain {
                 let title = format!(
-                    "── Page {} ──┼─ {} ─┼─ {} ─────",
+                    "─── Page {} ───┤ {} ├── {} ─────",
                     self.youtube_options.page(),
                     "Tab/Shift+Tab switch pages",
                     domain,
                 );
-                let props = table::TablePropsBuilder::from(props)
+                let props = TablePropsBuilder::from(props)
                     .with_title(title, tuirealm::tui::layout::Alignment::Left)
                     .with_header(&["Duration", "Name"])
-                    .with_widths(&[15, 85])
                     .with_table(table)
                     .build();
-                self.view
-                    .update(super::COMPONENT_SCROLLTABLE_YOUTUBE, props);
+                self.view.update(COMPONENT_TABLE_YOUTUBE, props);
             }
         }
     }
@@ -238,7 +228,7 @@ impl MainActivity {
         let tx = self.sender.clone();
 
         thread::spawn(move || -> Result<()> {
-            let _ = tx.send(super::TransferState::Running);
+            let _ = tx.send(TransferState::Running);
             // start download
             let download = ytd.download();
 
@@ -275,13 +265,15 @@ impl MainActivity {
                                                     lang_ext = ext2.to_string_lossy().to_string();
                                                 }
                                             }
-                                            let lyric_string = std::fs::read_to_string(f.path())?;
+                                            let lyric_string = std::fs::read_to_string(f.path());
                                             id3_tag.add_lyrics(Lyrics {
                                                 lang: "eng".to_string(),
                                                 description: lang_ext,
-                                                text: lyric_string,
+                                                text: lyric_string.unwrap_or_else(|_| {
+                                                    String::from("[00:00:01] No lyric")
+                                                }),
                                             });
-                                            std::fs::remove_file(f.path())?;
+                                            let _ = std::fs::remove_file(f.path());
                                         }
                                     }
                                 }
@@ -289,22 +281,22 @@ impl MainActivity {
 
                             let _ = id3_tag.write_to_path(&file_fullname, id3::Version::Id3v24);
 
-                            let _ = tx.send(super::TransferState::Success);
+                            let _ = tx.send(TransferState::Success);
                             sleep(Duration::from_secs(5));
-                            let _ = tx.send(super::TransferState::Completed(Some(file_fullname)));
+                            let _ = tx.send(TransferState::Completed(Some(file_fullname)));
                         }
                         Err(_) => {
                             // This shoudn't happen unless the output format of youtubedl changed
-                            let _ = tx.send(super::TransferState::Success);
+                            let _ = tx.send(TransferState::Success);
                             sleep(Duration::from_secs(5));
-                            let _ = tx.send(super::TransferState::Completed(None));
+                            let _ = tx.send(TransferState::Completed(None));
                         }
                     }
                 }
                 ResultType::IOERROR | ResultType::FAILURE => {
-                    let _ = tx.send(super::TransferState::ErrDownload);
+                    let _ = tx.send(TransferState::ErrDownload);
                     sleep(Duration::from_secs(5));
-                    let _ = tx.send(super::TransferState::Completed(None));
+                    let _ = tx.send(TransferState::Completed(None));
                 }
             }
             Ok(())
@@ -321,7 +313,6 @@ pub fn extract_filepath(output: &str, dir: &str) -> Result<String> {
     filename.push('/');
 
     if let Some(cap) = RE_FILENAME.captures(output) {
-        // filename.push_str(cap.name("name").unwrap().as_str());
         match cap.name("name") {
             Some(c) => filename.push_str(c.as_str()),
             None => bail!("parsing output error"),
@@ -338,7 +329,6 @@ mod tests {
 
     use crate::ui::activity::main::youtube_options::extract_filepath;
     use pretty_assertions::assert_eq;
-    // use
 
     #[test]
     fn test_youtube_output_parsing() {
