@@ -3,7 +3,7 @@
 //! `tageditor_activity` is the module which implements the Tageditor activity, which is the activity to
 //! edit tag and fetch lyrics
 
-mod lyric_options;
+mod songtag_options;
 /**
  * MIT License
  *
@@ -32,10 +32,9 @@ mod update;
 mod view;
 
 // Locals
+use super::{Activity, Context, ExitReason};
 use crate::song::Song;
 use crate::songtag::SongTag;
-// use super::super::super::player::Player;
-use super::{Activity, Context, ExitReason};
 // Ext
 use super::main::TransferState;
 use crossterm::terminal::enable_raw_mode;
@@ -45,6 +44,7 @@ use std::sync::mpsc::{Receiver, Sender};
 use tuirealm::View;
 
 // -- components
+const COMPONENT_TE_LABEL_HINT: &str = "LABEL_TE_HINT";
 const COMPONENT_TE_LABEL_HELP: &str = "LABEL_TE_HELP";
 const COMPONENT_TE_TEXT_HELP: &str = "TEXT_TE_HELP";
 const COMPONENT_TE_TEXT_ERROR: &str = "TEXT_TE_ERROR";
@@ -68,7 +68,7 @@ pub struct TagEditorActivity {
     view: View,               // View
     redraw: bool,
     song: Option<Song>,
-    lyric_options: Vec<SongTag>,
+    songtag_options: Vec<SongTag>,
     sender: Sender<TransferState>,
     receiver: Receiver<TransferState>,
     sender_songtag: Sender<SearchLyricState>,
@@ -95,7 +95,7 @@ impl Default for TagEditorActivity {
             view: View::init(),
             redraw: true, // Draw at first `on_draw`
             song: None,
-            lyric_options: vec![],
+            songtag_options: vec![],
             sender: tx,
             receiver: rx,
             sender_songtag: tx2,
