@@ -1,4 +1,3 @@
-use super::SearchLyricState;
 /**
  * MIT License
  *
@@ -23,94 +22,91 @@ use super::SearchLyricState;
  * SOFTWARE.
  */
 // locals
-use super::{TagEditorActivity, COMPONENT_TE_LABEL_HELP};
-use crate::ui::activity::main::{StatusLine, TransferState};
-use crate::ui::keymap::*;
-// use crate::ui::components::scrolltable;
-use super::ExitReason;
-use crate::song::Song;
-use crate::songtag;
+use super::{
+    ExitReason, SearchLyricState, TagEditorActivity, COMPONENT_TE_DELETE_LYRIC,
+    COMPONENT_TE_INPUT_ARTIST, COMPONENT_TE_INPUT_SONGNAME, COMPONENT_TE_LABEL_HELP,
+    COMPONENT_TE_RADIO_TAG, COMPONENT_TE_SCROLLTABLE_OPTIONS, COMPONENT_TE_SELECT_LYRIC,
+    COMPONENT_TE_TEXTAREA_LYRIC, COMPONENT_TE_TEXT_ERROR, COMPONENT_TE_TEXT_HELP,
+};
+use crate::{
+    song::Song,
+    songtag::songtag_search,
+    ui::activity::main::{StatusLine, TransferState},
+    ui::keymap::*,
+};
 use std::path::Path;
 use std::str::FromStr;
-use tui_realm_stdlib::label;
-use tuirealm::event::{Event, KeyCode, KeyEvent, KeyModifiers};
-use tuirealm::tui::style::Color;
-use tuirealm::PropsBuilder;
-use tuirealm::{Msg, Payload, Value};
+use tui_realm_stdlib::LabelPropsBuilder;
+use tuirealm::{
+    event::{Event, KeyCode, KeyEvent, KeyModifiers},
+    tui::style::Color,
+    Msg, Payload, PropsBuilder, Value,
+};
 
 impl TagEditorActivity {
     /// ### update
     ///
     /// Update auth activity model based on msg
     /// The function exits when returns None
-    pub(super) fn update(&mut self, msg: Option<(String, Msg)>) -> Option<(String, Msg)> {
+    pub fn update(&mut self, msg: Option<(String, Msg)>) -> Option<(String, Msg)> {
         let ref_msg: Option<(&str, &Msg)> = msg.as_ref().map(|(s, msg)| (s.as_str(), msg));
         match ref_msg {
             None => None, // Exit after None
             Some(msg) => match msg {
-                (super::COMPONENT_TE_RADIO_TAG, key) if key == &MSG_KEY_TAB => {
-                    self.view.active(super::COMPONENT_TE_INPUT_ARTIST);
+                (COMPONENT_TE_INPUT_ARTIST, key) if key == &MSG_KEY_TAB => {
+                    self.view.active(COMPONENT_TE_INPUT_SONGNAME);
                     None
                 }
-                (super::COMPONENT_TE_INPUT_ARTIST, key) if key == &MSG_KEY_TAB => {
-                    self.view.active(super::COMPONENT_TE_INPUT_SONGNAME);
-                    None
-                }
-                (super::COMPONENT_TE_INPUT_SONGNAME, key) if key == &MSG_KEY_TAB => {
-                    self.view.active(super::COMPONENT_TE_SCROLLTABLE_OPTIONS);
+                (COMPONENT_TE_INPUT_SONGNAME, key) if key == &MSG_KEY_TAB => {
+                    self.view.active(COMPONENT_TE_RADIO_TAG);
                     None
                 }
 
-                (super::COMPONENT_TE_SCROLLTABLE_OPTIONS, key) if key == &MSG_KEY_TAB => {
-                    self.view.active(super::COMPONENT_TE_SELECT_LYRIC);
+                (COMPONENT_TE_RADIO_TAG, key) if key == &MSG_KEY_TAB => {
+                    self.view.active(COMPONENT_TE_SCROLLTABLE_OPTIONS);
                     None
                 }
 
-                (super::COMPONENT_TE_SELECT_LYRIC, key) if key == &MSG_KEY_TAB => {
-                    self.view.active(super::COMPONENT_TE_DELETE_LYRIC);
+                (COMPONENT_TE_SCROLLTABLE_OPTIONS, key) if key == &MSG_KEY_TAB => {
+                    self.view.active(COMPONENT_TE_SELECT_LYRIC);
                     None
                 }
 
-                (super::COMPONENT_TE_DELETE_LYRIC, key) if key == &MSG_KEY_TAB => {
-                    self.view.active(super::COMPONENT_TE_TEXTAREA_LYRIC);
+                (COMPONENT_TE_SELECT_LYRIC, key) if key == &MSG_KEY_TAB => {
+                    self.view.active(COMPONENT_TE_DELETE_LYRIC);
                     None
                 }
 
-                (super::COMPONENT_TE_TEXTAREA_LYRIC, key) if key == &MSG_KEY_TAB => {
-                    self.view.active(super::COMPONENT_TE_RADIO_TAG);
+                (COMPONENT_TE_DELETE_LYRIC, key) if key == &MSG_KEY_TAB => {
+                    self.view.active(COMPONENT_TE_TEXTAREA_LYRIC);
                     None
                 }
-                (
-                    super::COMPONENT_TE_RADIO_TAG,
-                    Msg::OnSubmit(Payload::One(Value::Usize(choice))),
-                ) => {
+
+                (COMPONENT_TE_TEXTAREA_LYRIC, key) if key == &MSG_KEY_TAB => {
+                    self.view.active(COMPONENT_TE_INPUT_ARTIST);
+                    None
+                }
+                (COMPONENT_TE_RADIO_TAG, Msg::OnSubmit(Payload::One(Value::Usize(choice)))) => {
                     if *choice == 0 {
                         // Rename file by Tag
                         if let Some(mut song) = self.song.clone() {
                             if let Some(Payload::One(Value::Str(artist))) =
-                                self.view.get_state(super::COMPONENT_TE_INPUT_ARTIST)
+                                self.view.get_state(COMPONENT_TE_INPUT_ARTIST)
                             {
                                 song.artist = Some(artist);
                             }
                             if let Some(Payload::One(Value::Str(title))) =
-                                self.view.get_state(super::COMPONENT_TE_INPUT_SONGNAME)
+                                self.view.get_state(COMPONENT_TE_INPUT_SONGNAME)
                             {
                                 song.title = Some(title);
                             }
-                            match song.save() {
+                            match song.save_tag() {
                                 Ok(()) => {
-                                    match song.rename_by_tag() {
-                                        Ok(()) => {
-                                            if let Some(file) = song.file() {
-                                                self.exit_reason =
-                                                    Some(ExitReason::NeedRefreshPlaylist(
-                                                        file.to_string(),
-                                                    ));
-                                            }
-                                            self.init_by_song(&song)
-                                        }
-                                        Err(e) => self.mount_error(&e.to_string()),
-                                    };
+                                    if let Some(file) = song.file() {
+                                        self.exit_reason =
+                                            Some(ExitReason::NeedRefreshPlaylist(file.to_string()));
+                                    }
+                                    self.init_by_song(&song)
                                 }
                                 Err(e) => self.mount_error(&e.to_string()),
                             };
@@ -118,25 +114,23 @@ impl TagEditorActivity {
                     }
                     None
                 }
-                (super::COMPONENT_TE_SCROLLTABLE_OPTIONS, key)
+                (COMPONENT_TE_SCROLLTABLE_OPTIONS, key)
                     if (key == &MSG_KEY_CHAR_L) | (key == &MSG_KEY_ENTER) =>
                 {
-                    match self.view.get_state(super::COMPONENT_TE_SCROLLTABLE_OPTIONS) {
+                    match self.view.get_state(COMPONENT_TE_SCROLLTABLE_OPTIONS) {
                         Some(Payload::One(Value::Usize(index))) => {
-                            if self.lyric_options.is_empty() {
+                            if self.songtag_options.is_empty() {
                                 return None;
                             }
                             if let Some(mut song) = self.song.clone() {
-                                let song_tag = self.lyric_options.get(index)?;
+                                let song_tag = self.songtag_options.get(index)?;
                                 let lang_ext = song_tag
                                     .lang_ext
                                     .to_owned()
                                     .unwrap_or_else(|| String::from("eng"));
-                                let mut artist = String::from("");
-                                for a in song_tag.artist.iter() {
-                                    artist += a;
+                                if let Some(artist) = &song_tag.artist {
+                                    song.artist = Some(artist.to_owned());
                                 }
-                                song.artist = Some(artist);
                                 if let Some(title) = &song_tag.title {
                                     song.title = Some(title.to_owned());
                                 }
@@ -146,26 +140,21 @@ impl TagEditorActivity {
 
                                 if let Ok(lyric_string) = song_tag.fetch_lyric() {
                                     song.set_lyric(&lyric_string, &lang_ext);
-                                    if let Ok(artwork) = song_tag.fetch_photo() {
-                                        song.set_photo(artwork);
-                                    }
-                                    match song.save() {
-                                        Ok(()) => {
-                                            match song.rename_by_tag() {
-                                                Ok(()) => {
-                                                    if let Some(file) = song.file() {
-                                                        self.exit_reason =
-                                                            Some(ExitReason::NeedRefreshPlaylist(
-                                                                file.to_string(),
-                                                            ));
-                                                    }
-                                                    self.init_by_song(&song)
-                                                }
-                                                Err(e) => self.mount_error(&e.to_string()),
-                                            };
+                                }
+                                if let Ok(artwork) = song_tag.fetch_photo() {
+                                    song.set_photo(artwork);
+                                }
+
+                                match song.save_tag() {
+                                    Ok(()) => {
+                                        if let Some(file) = song.file() {
+                                            self.exit_reason = Some(
+                                                ExitReason::NeedRefreshPlaylist(file.to_string()),
+                                            );
                                         }
-                                        Err(e) => self.mount_error(&e.to_string()),
+                                        self.init_by_song(&song)
                                     }
+                                    Err(e) => self.mount_error(&e.to_string()),
                                 }
                             }
 
@@ -176,11 +165,11 @@ impl TagEditorActivity {
                 }
 
                 // download
-                (super::COMPONENT_TE_SCROLLTABLE_OPTIONS, key) if key == &MSG_KEY_CHAR_S => {
+                (COMPONENT_TE_SCROLLTABLE_OPTIONS, key) if key == &MSG_KEY_CHAR_S => {
                     if let Some(Payload::One(Value::Usize(index))) =
-                        self.view.get_state(super::COMPONENT_TE_SCROLLTABLE_OPTIONS)
+                        self.view.get_state(COMPONENT_TE_SCROLLTABLE_OPTIONS)
                     {
-                        if let Some(song_tag) = self.lyric_options.get(index) {
+                        if let Some(song_tag) = self.songtag_options.get(index) {
                             if let Some(song) = &self.song {
                                 if let Some(file) = &song.file {
                                     if let Err(e) = song_tag.download(file, self.sender.clone()) {
@@ -194,21 +183,17 @@ impl TagEditorActivity {
                 }
 
                 // select lyric
-                (
-                    super::COMPONENT_TE_SELECT_LYRIC,
-                    Msg::OnSubmit(Payload::One(Value::Usize(index))),
-                ) => {
+                (COMPONENT_TE_SELECT_LYRIC, Msg::OnSubmit(Payload::One(Value::Usize(index)))) => {
                     if let Some(mut song) = self.song.clone() {
                         song.lyric_selected = *index;
                         self.init_by_song(&song);
-                        self.song = Some(song);
                     }
                     None
                 }
 
                 // delete lyric
-                (super::COMPONENT_TE_DELETE_LYRIC, key) if key == &MSG_KEY_ENTER => {
-                    if let Some(mut song) = self.song.take() {
+                (COMPONENT_TE_DELETE_LYRIC, key) if key == &MSG_KEY_ENTER => {
+                    if let Some(mut song) = self.song.clone() {
                         if song.lyric_frames.is_empty() {
                             song.parsed_lyric = None;
                             return None;
@@ -219,33 +204,27 @@ impl TagEditorActivity {
                         {
                             song.lyric_selected -= 1;
                         }
-                        match song.save() {
-                            Ok(_) => {
-                                self.init_by_song(&song);
-                                self.song = Some(song);
-                            }
+                        match song.save_tag() {
+                            Ok(_) => self.init_by_song(&song),
                             Err(e) => self.mount_error(&e.to_string()),
                         }
                     }
                     None
                 }
 
-                (super::COMPONENT_TE_INPUT_ARTIST, Msg::OnSubmit(Payload::One(Value::Str(_))))
-                | (
-                    super::COMPONENT_TE_INPUT_SONGNAME,
-                    Msg::OnSubmit(Payload::One(Value::Str(_))),
-                ) => {
+                (COMPONENT_TE_INPUT_ARTIST, Msg::OnSubmit(Payload::One(Value::Str(_))))
+                | (COMPONENT_TE_INPUT_SONGNAME, Msg::OnSubmit(Payload::One(Value::Str(_)))) => {
                     // Get Tag
                     let mut search_str = String::new();
                     if let Some(Payload::One(Value::Str(artist))) =
-                        self.view.get_state(super::COMPONENT_TE_INPUT_ARTIST)
+                        self.view.get_state(COMPONENT_TE_INPUT_ARTIST)
                     {
                         search_str.push_str(&artist);
                     }
 
                     search_str.push(' ');
                     if let Some(Payload::One(Value::Str(title))) =
-                        self.view.get_state(super::COMPONENT_TE_INPUT_SONGNAME)
+                        self.view.get_state(COMPONENT_TE_INPUT_SONGNAME)
                     {
                         search_str.push_str(&title);
                     }
@@ -261,11 +240,11 @@ impl TagEditorActivity {
                         }
                     }
 
-                    songtag::lyric_options(&search_str, self.sender_songtag.clone());
+                    songtag_search(&search_str, self.sender_songtag.clone());
                     None
                 }
 
-                (super::COMPONENT_TE_SCROLLTABLE_OPTIONS, key) if key == &MSG_KEY_CHAR_G => {
+                (COMPONENT_TE_SCROLLTABLE_OPTIONS, key) if key == &MSG_KEY_CHAR_G => {
                     let event: Event = Event::Key(KeyEvent {
                         code: KeyCode::Home,
                         modifiers: KeyModifiers::NONE,
@@ -274,9 +253,7 @@ impl TagEditorActivity {
                     None
                 }
 
-                (super::COMPONENT_TE_SCROLLTABLE_OPTIONS, key)
-                    if key == &MSG_KEY_CHAR_CAPITAL_G =>
-                {
+                (COMPONENT_TE_SCROLLTABLE_OPTIONS, key) if key == &MSG_KEY_CHAR_CAPITAL_G => {
                     let event: Event = Event::Key(KeyEvent {
                         code: KeyCode::End,
                         modifiers: KeyModifiers::NONE,
@@ -285,7 +262,7 @@ impl TagEditorActivity {
                     None
                 }
 
-                (super::COMPONENT_TE_TEXTAREA_LYRIC, key) if key == &MSG_KEY_CHAR_G => {
+                (COMPONENT_TE_TEXTAREA_LYRIC, key) if key == &MSG_KEY_CHAR_G => {
                     let event: Event = Event::Key(KeyEvent {
                         code: KeyCode::Home,
                         modifiers: KeyModifiers::NONE,
@@ -294,7 +271,7 @@ impl TagEditorActivity {
                     None
                 }
 
-                (super::COMPONENT_TE_TEXTAREA_LYRIC, key) if key == &MSG_KEY_CHAR_CAPITAL_G => {
+                (COMPONENT_TE_TEXTAREA_LYRIC, key) if key == &MSG_KEY_CHAR_CAPITAL_G => {
                     let event: Event = Event::Key(KeyEvent {
                         code: KeyCode::End,
                         modifiers: KeyModifiers::NONE,
@@ -303,13 +280,13 @@ impl TagEditorActivity {
                     None
                 }
 
-                (_, key) if key == &MSG_KEY_QUESTION_MARK => {
+                (_, key) if key == &MSG_KEY_CTRL_H => {
                     // Show help
                     self.mount_help();
                     None
                 }
                 // -- help
-                (super::COMPONENT_TE_TEXT_HELP, key)
+                (COMPONENT_TE_TEXT_HELP, key)
                     if (key == &MSG_KEY_ENTER) | (key == &MSG_KEY_ESC) =>
                 {
                     self.umount_help();
@@ -352,7 +329,7 @@ impl TagEditorActivity {
                 }
 
                 // -- error
-                (super::COMPONENT_TE_TEXT_ERROR, key)
+                (COMPONENT_TE_TEXT_ERROR, key)
                     if (key == &MSG_KEY_ESC) | (key == &MSG_KEY_ENTER) =>
                 {
                     self.umount_error();
@@ -361,7 +338,7 @@ impl TagEditorActivity {
 
                 (_, key) if (key == &MSG_KEY_ESC) | (key == &MSG_KEY_CHAR_CAPITAL_Q) => {
                     // Quit on esc
-                    self.exit_reason = Some(super::ExitReason::Quit);
+                    self.exit_reason = Some(ExitReason::Quit);
                     None
                 }
                 _ => None,
@@ -381,10 +358,8 @@ impl TagEditorActivity {
                 TransferState::Completed(file) => {
                     if let Some(f) = file {
                         if let Ok(song) = Song::from_str(&f) {
-                            let song1 = song.to_owned();
-                            self.song = Some(song);
                             self.exit_reason = Some(ExitReason::NeedRefreshPlaylist(f));
-                            self.init_by_song(&song1);
+                            self.init_by_song(&song);
                         }
                     }
                     self.update_status_line(StatusLine::Default);
@@ -403,17 +378,17 @@ impl TagEditorActivity {
 
     pub fn update_lyric_options(&mut self) {
         if let Ok(SearchLyricState::Finish(l)) = self.receiver_songtag.try_recv() {
-            self.add_lyric_options(l);
+            self.add_songtag_options(l);
             self.redraw = true;
         }
     }
     pub fn update_status_line(&mut self, s: StatusLine) {
         match s {
             StatusLine::Default => {
-                let text = "Press \"?\" for help.".to_string();
+                let text = "Press <CTRL+H> for help.".to_string();
 
                 if let Some(props) = self.view.get_props(COMPONENT_TE_LABEL_HELP) {
-                    let props = label::LabelPropsBuilder::from(props)
+                    let props = LabelPropsBuilder::from(props)
                         .with_background(Color::Reset)
                         .with_foreground(Color::Cyan)
                         .with_text(text)
@@ -434,7 +409,7 @@ impl TagEditorActivity {
                 let text = " Downloading...".to_string();
 
                 if let Some(props) = self.view.get_props(COMPONENT_TE_LABEL_HELP) {
-                    let props = label::LabelPropsBuilder::from(props)
+                    let props = LabelPropsBuilder::from(props)
                         .with_text(text)
                         .with_foreground(Color::White)
                         .with_background(Color::Red)
@@ -449,7 +424,7 @@ impl TagEditorActivity {
                 let text = " Download Success!".to_string();
 
                 if let Some(props) = self.view.get_props(COMPONENT_TE_LABEL_HELP) {
-                    let props = label::LabelPropsBuilder::from(props)
+                    let props = LabelPropsBuilder::from(props)
                         .with_text(text)
                         .with_foreground(Color::Black)
                         .with_background(Color::Green)
@@ -464,7 +439,7 @@ impl TagEditorActivity {
                 let text = " Download Error!".to_string();
 
                 if let Some(props) = self.view.get_props(COMPONENT_TE_LABEL_HELP) {
-                    let props = label::LabelPropsBuilder::from(props)
+                    let props = LabelPropsBuilder::from(props)
                         .with_text(text)
                         .with_foreground(Color::White)
                         .with_background(Color::Red)
