@@ -22,25 +22,41 @@
  * SOFTWARE.
  */
 // Locals
-use super::TagEditorActivity;
-use crate::song::Song;
-use crate::ui::components::msgbox::{MsgBox, MsgBoxPropsBuilder};
-use crate::ui::components::{counter, table};
-use crate::ui::draw_area_in;
+use super::{
+    TagEditorActivity, COMPONENT_TE_DELETE_LYRIC, COMPONENT_TE_INPUT_ARTIST,
+    COMPONENT_TE_INPUT_SONGNAME, COMPONENT_TE_LABEL_HELP, COMPONENT_TE_LABEL_HINT,
+    COMPONENT_TE_RADIO_TAG, COMPONENT_TE_SCROLLTABLE_OPTIONS, COMPONENT_TE_SELECT_LYRIC,
+    COMPONENT_TE_TEXTAREA_LYRIC, COMPONENT_TE_TEXT_ERROR, COMPONENT_TE_TEXT_HELP,
+};
+use crate::{
+    song::Song,
+    ui::{
+        components::{
+            counter,
+            msgbox::{MsgBox, MsgBoxPropsBuilder},
+        },
+        draw_area_in,
+    },
+};
 // Ext
 use tui_realm_stdlib::{
-    input, label, radio, select,
-    table::{Table, TablePropsBuilder},
-    textarea,
+    Input, InputPropsBuilder, Label, LabelPropsBuilder, Radio, RadioPropsBuilder, Select,
+    SelectPropsBuilder, Table, TablePropsBuilder, Textarea, TextareaPropsBuilder,
+};
+use tuirealm::{
+    props::{
+        borders::{BorderType, Borders},
+        TableBuilder, TextSpan,
+    },
+    tui::{
+        layout::{Alignment, Constraint, Direction, Layout},
+        style::Color,
+        widgets::Clear,
+    },
+    PropsBuilder, View,
 };
-use tuirealm::props::borders::{BorderType, Borders};
-use tuirealm::props::{TableBuilder, TextSpan};
-use tuirealm::{PropsBuilder, View};
 
 // tui
-use tuirealm::tui::layout::{Alignment, Constraint, Direction, Layout};
-use tuirealm::tui::style::Color;
-use tuirealm::tui::widgets::Clear;
 
 impl TagEditorActivity {
     // -- view
@@ -53,19 +69,29 @@ impl TagEditorActivity {
         self.view = View::init();
         // Let's mount the component we need
         self.view.mount(
-            super::COMPONENT_TE_LABEL_HELP,
-            Box::new(label::Label::new(
-                label::LabelPropsBuilder::default()
-                    .with_foreground(Color::Cyan)
-                    .with_text(String::from("Press \"?\" for help."))
+            COMPONENT_TE_LABEL_HINT,
+            Box::new(Label::new(
+                LabelPropsBuilder::default()
+                    .with_foreground(Color::Magenta)
+                    .with_text(String::from("Press <ENTER> to search:"))
                     .build(),
             )),
         );
 
         self.view.mount(
-            super::COMPONENT_TE_RADIO_TAG,
-            Box::new(radio::Radio::new(
-                radio::RadioPropsBuilder::default()
+            COMPONENT_TE_LABEL_HELP,
+            Box::new(Label::new(
+                LabelPropsBuilder::default()
+                    .with_foreground(Color::Cyan)
+                    .with_text(String::from("Press <CTRL+H> for help."))
+                    .build(),
+            )),
+        );
+
+        self.view.mount(
+            COMPONENT_TE_RADIO_TAG,
+            Box::new(Radio::new(
+                RadioPropsBuilder::default()
                     .with_color(Color::Magenta)
                     .with_borders(
                         Borders::BOTTOM | Borders::TOP,
@@ -81,9 +107,9 @@ impl TagEditorActivity {
         );
 
         self.view.mount(
-            super::COMPONENT_TE_INPUT_ARTIST,
-            Box::new(input::Input::new(
-                input::InputPropsBuilder::default()
+            COMPONENT_TE_INPUT_ARTIST,
+            Box::new(Input::new(
+                InputPropsBuilder::default()
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::LightYellow)
                     .with_foreground(Color::Cyan)
                     .with_label(String::from("Search Artist"), Alignment::Left)
@@ -91,9 +117,9 @@ impl TagEditorActivity {
             )),
         );
         self.view.mount(
-            super::COMPONENT_TE_INPUT_SONGNAME,
-            Box::new(input::Input::new(
-                input::InputPropsBuilder::default()
+            COMPONENT_TE_INPUT_SONGNAME,
+            Box::new(Input::new(
+                InputPropsBuilder::default()
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::LightYellow)
                     .with_foreground(Color::Cyan)
                     .with_label(String::from("Search Song"), Alignment::Left)
@@ -102,9 +128,9 @@ impl TagEditorActivity {
         );
         // Scrolltable
         self.view.mount(
-            super::COMPONENT_TE_SCROLLTABLE_OPTIONS,
-            Box::new(table::Table::new(
-                table::TablePropsBuilder::default()
+            COMPONENT_TE_SCROLLTABLE_OPTIONS,
+            Box::new(Table::new(
+                TablePropsBuilder::default()
                     .with_background(Color::Black)
                     .with_highlighted_str(Some("🚀"))
                     .with_highlighted_color(Color::LightBlue)
@@ -126,9 +152,9 @@ impl TagEditorActivity {
         );
         // Lyric Select
         self.view.mount(
-            super::COMPONENT_TE_SELECT_LYRIC,
-            Box::new(select::Select::new(
-                select::SelectPropsBuilder::default()
+            COMPONENT_TE_SELECT_LYRIC,
+            Box::new(Select::new(
+                SelectPropsBuilder::default()
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::LightRed)
                     .with_background(Color::Black)
                     .with_foreground(Color::LightRed)
@@ -141,7 +167,7 @@ impl TagEditorActivity {
 
         // Lyric Delete
         self.view.mount(
-            super::COMPONENT_TE_DELETE_LYRIC,
+            COMPONENT_TE_DELETE_LYRIC,
             Box::new(counter::Counter::new(
                 counter::CounterPropsBuilder::default()
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::LightRed)
@@ -153,9 +179,9 @@ impl TagEditorActivity {
 
         // Lyric Textarea
         self.view.mount(
-            super::COMPONENT_TE_TEXTAREA_LYRIC,
-            Box::new(textarea::Textarea::new(
-                textarea::TextareaPropsBuilder::default()
+            COMPONENT_TE_TEXTAREA_LYRIC,
+            Box::new(Textarea::new(
+                TextareaPropsBuilder::default()
                     .with_foreground(Color::Green)
                     .with_highlighted_str(Some("🚀"))
                     .with_max_scroll_step(4)
@@ -170,7 +196,7 @@ impl TagEditorActivity {
         );
 
         // We need to initialize the focus
-        self.view.active(super::COMPONENT_TE_RADIO_TAG);
+        self.view.active(COMPONENT_TE_INPUT_ARTIST);
     }
 
     /// View gui
@@ -183,7 +209,7 @@ impl TagEditorActivity {
                     .margin(0)
                     .constraints(
                         [
-                            Constraint::Length(4),
+                            Constraint::Length(1),
                             Constraint::Length(3),
                             Constraint::Min(2),
                             Constraint::Length(1),
@@ -222,52 +248,40 @@ impl TagEditorActivity {
                     .constraints([Constraint::Ratio(1, 2), Constraint::Ratio(1, 2)].as_ref())
                     .split(chunks_middle2_right[0]);
 
+                self.view.render(COMPONENT_TE_LABEL_HINT, f, chunks_main[0]);
                 self.view
-                    .render(super::COMPONENT_TE_RADIO_TAG, f, chunks_main[0]);
+                    .render(COMPONENT_TE_INPUT_ARTIST, f, chunks_middle1[0]);
                 self.view
-                    .render(super::COMPONENT_TE_INPUT_ARTIST, f, chunks_middle1[0]);
+                    .render(COMPONENT_TE_INPUT_SONGNAME, f, chunks_middle1[1]);
                 self.view
-                    .render(super::COMPONENT_TE_INPUT_SONGNAME, f, chunks_middle1[1]);
-                self.view.render(
-                    super::COMPONENT_TE_SCROLLTABLE_OPTIONS,
-                    f,
-                    chunks_middle2[0],
-                );
+                    .render(COMPONENT_TE_RADIO_TAG, f, chunks_middle1[2]);
                 self.view
-                    .render(super::COMPONENT_TE_LABEL_HELP, f, chunks_main[3]);
+                    .render(COMPONENT_TE_SCROLLTABLE_OPTIONS, f, chunks_middle2[0]);
+                self.view.render(COMPONENT_TE_LABEL_HELP, f, chunks_main[3]);
 
-                self.view.render(
-                    super::COMPONENT_TE_SELECT_LYRIC,
-                    f,
-                    chunks_middle2_right_top[0],
-                );
-                self.view.render(
-                    super::COMPONENT_TE_DELETE_LYRIC,
-                    f,
-                    chunks_middle2_right_top[1],
-                );
+                self.view
+                    .render(COMPONENT_TE_SELECT_LYRIC, f, chunks_middle2_right_top[0]);
+                self.view
+                    .render(COMPONENT_TE_DELETE_LYRIC, f, chunks_middle2_right_top[1]);
 
-                self.view.render(
-                    super::COMPONENT_TE_TEXTAREA_LYRIC,
-                    f,
-                    chunks_middle2_right[1],
-                );
+                self.view
+                    .render(COMPONENT_TE_TEXTAREA_LYRIC, f, chunks_middle2_right[1]);
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_TE_TEXT_ERROR) {
+                if let Some(props) = self.view.get_props(COMPONENT_TE_TEXT_ERROR) {
                     if props.visible {
                         let popup = draw_area_in(f.size(), 50, 10);
                         f.render_widget(Clear, popup);
                         // make popup
-                        self.view.render(super::COMPONENT_TE_TEXT_ERROR, f, popup);
+                        self.view.render(COMPONENT_TE_TEXT_ERROR, f, popup);
                     }
                 }
 
-                if let Some(props) = self.view.get_props(super::COMPONENT_TE_TEXT_HELP) {
+                if let Some(props) = self.view.get_props(COMPONENT_TE_TEXT_HELP) {
                     if props.visible {
                         // make popup
                         let popup = draw_area_in(f.size(), 50, 70);
                         f.render_widget(Clear, popup);
-                        self.view.render(super::COMPONENT_TE_TEXT_HELP, f, popup);
+                        self.view.render(COMPONENT_TE_TEXT_HELP, f, popup);
                     }
                 }
             });
@@ -283,7 +297,7 @@ impl TagEditorActivity {
     pub(super) fn mount_error(&mut self, text: &str) {
         // Mount
         self.view.mount(
-            super::COMPONENT_TE_TEXT_ERROR,
+            COMPONENT_TE_TEXT_ERROR,
             Box::new(MsgBox::new(
                 MsgBoxPropsBuilder::default()
                     .with_foreground(Color::Red)
@@ -294,60 +308,60 @@ impl TagEditorActivity {
             )),
         );
         // Give focus to error
-        self.view.active(super::COMPONENT_TE_TEXT_ERROR);
+        self.view.active(COMPONENT_TE_TEXT_ERROR);
     }
 
     /// ### umount_error
     ///
     /// Umount error message
     pub(super) fn umount_error(&mut self) {
-        self.view.umount(super::COMPONENT_TE_TEXT_ERROR);
+        self.view.umount(COMPONENT_TE_TEXT_ERROR);
     }
 
     // initialize the value in tageditor based on info from Song
     pub fn init_by_song(&mut self, s: &Song) {
-        self.song = Some(s.clone());
-        if let Some(props) = self.view.get_props(super::COMPONENT_TE_INPUT_ARTIST) {
-            if let Some(artist) = s.artist() {
-                let props = input::InputPropsBuilder::from(props)
+        self.song = Some(s.to_owned());
+        if let Some(artist) = s.artist() {
+            if let Some(props) = self.view.get_props(COMPONENT_TE_INPUT_ARTIST) {
+                let props = InputPropsBuilder::from(props)
                     .with_value(artist.to_string())
                     .build();
-                self.view.update(super::COMPONENT_TE_INPUT_ARTIST, props);
+                self.view.update(COMPONENT_TE_INPUT_ARTIST, props);
             }
         }
 
-        if let Some(props) = self.view.get_props(super::COMPONENT_TE_INPUT_SONGNAME) {
-            if let Some(title) = s.title() {
-                let props = input::InputPropsBuilder::from(props)
+        if let Some(title) = s.title() {
+            if let Some(props) = self.view.get_props(COMPONENT_TE_INPUT_SONGNAME) {
+                let props = InputPropsBuilder::from(props)
                     .with_value(title.to_string())
                     .build();
-                self.view.update(super::COMPONENT_TE_INPUT_SONGNAME, props);
+                self.view.update(COMPONENT_TE_INPUT_SONGNAME, props);
             }
         }
 
         if s.lyric_frames.is_empty() {
-            if let Some(props) = self.view.get_props(super::COMPONENT_TE_SELECT_LYRIC) {
-                let props = select::SelectPropsBuilder::from(props)
+            if let Some(props) = self.view.get_props(COMPONENT_TE_SELECT_LYRIC) {
+                let props = SelectPropsBuilder::from(props)
                     .with_options(&["Empty"])
                     .build();
-                let msg = self.view.update(super::COMPONENT_TE_SELECT_LYRIC, props);
+                let msg = self.view.update(COMPONENT_TE_SELECT_LYRIC, props);
                 self.update(msg);
             }
 
-            if let Some(props) = self.view.get_props(super::COMPONENT_TE_DELETE_LYRIC) {
+            if let Some(props) = self.view.get_props(COMPONENT_TE_DELETE_LYRIC) {
                 let props = counter::CounterPropsBuilder::from(props)
                     .with_value(0)
                     .build();
-                let msg = self.view.update(super::COMPONENT_TE_DELETE_LYRIC, props);
+                let msg = self.view.update(COMPONENT_TE_DELETE_LYRIC, props);
                 self.update(msg);
             }
 
-            if let Some(props) = self.view.get_props(super::COMPONENT_TE_TEXTAREA_LYRIC) {
-                let props = textarea::TextareaPropsBuilder::from(props)
+            if let Some(props) = self.view.get_props(COMPONENT_TE_TEXTAREA_LYRIC) {
+                let props = TextareaPropsBuilder::from(props)
                     .with_title("Empty Lyrics".to_string(), Alignment::Left)
                     .with_texts(vec![TextSpan::new("No Lyrics.")])
                     .build();
-                let msg = self.view.update(super::COMPONENT_TE_TEXTAREA_LYRIC, props);
+                let msg = self.view.update(COMPONENT_TE_TEXTAREA_LYRIC, props);
                 self.update(msg);
             }
 
@@ -360,19 +374,19 @@ impl TagEditorActivity {
         }
         vec_lang.sort();
 
-        if let Some(props) = self.view.get_props(super::COMPONENT_TE_SELECT_LYRIC) {
-            let props = select::SelectPropsBuilder::from(props)
+        if let Some(props) = self.view.get_props(COMPONENT_TE_SELECT_LYRIC) {
+            let props = SelectPropsBuilder::from(props)
                 .with_options(&vec_lang)
                 .build();
-            let msg = self.view.update(super::COMPONENT_TE_SELECT_LYRIC, props);
+            let msg = self.view.update(COMPONENT_TE_SELECT_LYRIC, props);
             self.update(msg);
         }
 
-        if let Some(props) = self.view.get_props(super::COMPONENT_TE_DELETE_LYRIC) {
+        if let Some(props) = self.view.get_props(COMPONENT_TE_DELETE_LYRIC) {
             let props = counter::CounterPropsBuilder::from(props)
                 .with_value(vec_lang.len())
                 .build();
-            let msg = self.view.update(super::COMPONENT_TE_DELETE_LYRIC, props);
+            let msg = self.view.update(COMPONENT_TE_DELETE_LYRIC, props);
             self.update(msg);
         }
 
@@ -383,30 +397,33 @@ impl TagEditorActivity {
             }
         }
 
-        if let Some(props) = self.view.get_props(super::COMPONENT_TE_TEXTAREA_LYRIC) {
-            let props = textarea::TextareaPropsBuilder::from(props)
+        if let Some(props) = self.view.get_props(COMPONENT_TE_TEXTAREA_LYRIC) {
+            let props = TextareaPropsBuilder::from(props)
                 .with_title(
                     format!("{} Lyrics:", vec_lang[s.lyric_selected as usize]),
                     Alignment::Left,
                 )
                 .with_texts(vec_lyric)
                 .build();
-            let msg = self.view.update(super::COMPONENT_TE_TEXTAREA_LYRIC, props);
+            let msg = self.view.update(COMPONENT_TE_TEXTAREA_LYRIC, props);
             self.update(msg);
         }
     }
 
     pub(super) fn mount_help(&mut self) {
         self.view.mount(
-            super::COMPONENT_TE_TEXT_HELP,
+            COMPONENT_TE_TEXT_HELP,
             Box::new(Table::new(
                 TablePropsBuilder::default()
                     .with_borders(Borders::ALL, BorderType::Rounded, Color::Green)
                     .with_title("Help", Alignment::Center)
                     .with_header(&["Key", "Function"])
-                    .with_widths(&[35, 65])
+                    .with_widths(&[25, 75])
                     .with_table(
                         TableBuilder::default()
+                            .add_col(TextSpan::new("<ENTER>").bold().fg(Color::Cyan))
+                            .add_col(TextSpan::from("Search when focus Artist or Song name."))
+                            .add_row()
                             .add_col(TextSpan::new("<ESC> or <Q>").bold().fg(Color::Cyan))
                             .add_col(TextSpan::from("Exit"))
                             .add_row()
@@ -416,11 +433,8 @@ impl TagEditorActivity {
                             .add_col(TextSpan::new("<h,j,k,l,g,G>").bold().fg(Color::Cyan))
                             .add_col(TextSpan::from("Move cursor(vim style)"))
                             .add_row()
-                            .add_col(TextSpan::new("<enter>").bold().fg(Color::Cyan))
-                            .add_col(TextSpan::from("in editor start search"))
-                            .add_row()
-                            .add_col(TextSpan::new("<enter/l>").bold().fg(Color::Cyan))
-                            .add_col(TextSpan::from("Embed Lyrics"))
+                            .add_col(TextSpan::new("<ENTER/l>").bold().fg(Color::Cyan))
+                            .add_col(TextSpan::from("Embed selected songtag."))
                             .add_row()
                             .add_col(TextSpan::new("<s>").bold().fg(Color::Cyan))
                             .add_col(TextSpan::from("Download selected song"))
@@ -430,13 +444,13 @@ impl TagEditorActivity {
             )),
         );
         // Active help
-        self.view.active(super::COMPONENT_TE_TEXT_HELP);
+        self.view.active(COMPONENT_TE_TEXT_HELP);
     }
 
     /// ### umount_help
     ///
     /// Umount help
     pub(super) fn umount_help(&mut self) {
-        self.view.umount(super::COMPONENT_TE_TEXT_HELP);
+        self.view.umount(COMPONENT_TE_TEXT_HELP);
     }
 }
